from . import string
import traceback

class Tester:
	@property
	def classname(self):
		return self.__class__.__name__
	
	@classmethod
	def do(cls):
		c = cls()
		classname = c.classname
		retvals = []

		for testsection in filter(lambda k : not k in dir(Tester), dir(cls)):
			for f in getattr(c,testsection)():
				retval_el = [classname,testsection,f.__name__]
				try:

					
					resp = f()
					if resp is not None:
						retval_el += ["Failed",resp]
					else:
						retval_el += ["Succes",""]

					retvals.append(retval_el)
				except:
					retval_el += ["Failed",traceback.format_exc()]

		for a in string.same_len_ar(retvals):
			print(a)



from jivi.pr.Object import Object

from .TableAndField import Table,Field

class Session(Object):
	attr = []
	attr.append(dict(
		n='ppc',
		r=1,
		a='pixels-per-column'
		))
	attr.append(dict(n='ppr',r=1,a='pixels-per-row'))
	attr.append(dict(n='td',r=1,a='temp-directory',d=r'C:\temp'))
	
	def __init__(self,env,tabs):

		Object.__init__(self,env=env,tab={a['ck'] : a['cv'] for a in tabs['session']})
		
		self.tables = {}
		self.indexes = {}
		file_by_recid = {}
		self.font_lenght = {}
		
		self.ppr = int(self.ppr)
		self.ppc = int(self.ppc)
		
		#fi cc cl pl
		for a in tabs['font']:
			fi = a['fi']
			cc = a['cc']
			
			pl = a['pl']
			if not fi in self.font_lenght:
				self.font_lenght[fi] = {}
			self.font_lenght[fi][cc] = [pl, self.ptc(pl)]



		def check_tab(tab,required):
			tab = {k.lower().strip() : v for k,v in t.items()}
			for a in required:
				if not a in tab: return 0
			return tab
		for t in tabs['file']:
			tab = check_tab(t,['rrecid'])
			if not tab: continue
			table = Table(env=self.env,tab=tab)
			self.tables[table.name] = table
			file_by_recid[tab['rrecid']] = table
		for t in tabs['field']:
			tab = check_tab(t,['rrecid','_file-recid'])
			field = Field(env=self.env,tab=tab,table=file_by_recid[tab['_file-recid']])
				
				
				
	def ptc(self,x,dontconv=0):
		try:
			f = float(x)
		except:
			return None
		if dontconv: return f
		return f / self.ppc
			
	def htp(self,x,dontconv=0):
		try:
			f = float(x)
		except:
			return None
		if dontconv: return f
		return self.ppr * f
		
	def wtp(self,x,dontconv=0):
		try:
			f = float(x)
		except:
			return None
		if dontconv: return f
		return self.ppc * f

	def ctp(self,x,dontconv=0):
		try:
			f = float(x)
		except:
			return None
		if dontconv: return f
		return self.ppc * ( f - 1)

	def rtp(self,x,dontconv=0):
		try:
			f = float(x)
		except:
			return None
		if dontconv: return f
		return self.ppr * ( f - 1)


from Com import Com

class Viewas:
	def __init__(self,ob):
		self.ob = ob
	
	
	
class Combobox(Viewas):
	def __init__(self,*a,**b):
		Viewas.__init__(self,*a,**b)
		

class Fillin(Viewas):
	def __init__(self,*a,**b):
		Viewas.__init__(self,*a,**b)
		

class Radioset(Viewas):
	def __init__(self,*a,**b):
		Viewas.__init__(self,*a,**b)
		

class RadioButtons(Viewas):
	def __init__(self,*a,**b):
		Viewas.__init__(self,*a,**b)
		

class Selectionlist(Viewas):
	def __init__(self,*a,**b):
		Viewas.__init__(self,*a,**b)
		

class Slider(Viewas):
	def __init__(self,*a,**b):
		Viewas.__init__(self,*a,**b)
		

class Text(Viewas):
	def __init__(self,*a,**b):
		Viewas.__init__(self,*a,**b)
		

class Togglebox(Viewas):
	def __init__(self,*a,**b):
		Viewas.__init__(self,*a,**b)
		
class Editor(Viewas):
	def __init__(self,*a,**b):
		Viewas.__init__(self,*a,**b)
		

class Button(Viewas):
	def __init__(self,*a,**b):
		Viewas.__init__(self,*a,**b)
		


class Browse(Viewas):
	def __init__(self,*a,**b):
		Viewas.__init__(self,*a,**b)
		


"""
EFINE [ PRIVATE ] IMAGE image-name
  {image-phrase | LIKE image | size-phrase}
  [ BGCOLOR expression]
  [ FGCOLOR expression]
  [ CONVERT-3D-COLORS ]
  [ TOOLTIP tooltip]
  [ STRETCH-TO-FIT [ RETAIN-SHAPE ]] [ TRANSPARENT ]
  """
  
  
class Image(Viewas):
	def __init__(self,*a,**b):
		Viewas.__init__(self,*a,**b)
		



class Frame(Viewas):
	def __init__(self,*a,**b):
		Viewas.__init__(self,*a,**b)
		



viewas_tab = {'button' : Button,'radio-buttons' : RadioButtons,'editor' : Editor,'combo-box' : Combobox,'fill-in' : Fillin,'radio-set' : Radioset,'selection-list' : Selectionlist,'slider' : Slider,'text' : Text,'toggle-box' : Togglebox}
def viewas(ob):
	if not ob.has.view_as: return
	if not ob.view_as: return
	va = [a for a in ob.view_as.replace("\n"," ").split(' ') if a]
	if not va: return
	if va[0].lower().strip() == 'view-as':
		va = va[1:]
	if not va: return
	type = va[0].lower().strip()
	va = va[1:]
	if not va: return
	
	
	
	
	if not type in viewas_tab:
		print(type)
		print('view_as_tab')
		return
	return view_as_tab[type](ob)


from jivi.fs import *
for k,v in viewas_tab.items():
	File.write("view_as\\%s.html" % v.__name__,"<div> </div>")
"""

VIEW-AS COMBO-BOX
  [LIST-ITEMS value [,value] ... |
   LIST-ITEM-PAIRS label,value [,label,value] ...]
  [{SIZE-PIXELS | SIZE-CHARS | SIZE} width BY height]
  [INNER-LINES height]
  [SORT] [SUBTYPE]
  [TOOLTIP <tooltip>]
  
  VIEW-AS EDITOR
  {{SIZE-PIXELS | SIZE-CHARS | SIZE} width BY height |
   INNER-CHARS num-chars INNER-LINES num-lines}
  [SCROLLBAR-HORIZONTAL] [SCROLLBAR-VERTICAL]
  [MAX-CHARS chars] [NO-WORD-WRAP] [LARGE]
  [BUFFER-CHARS chars] [BUFFER-LINES lines]
  [NO-BOX]
  [TOOLTIP <tooltip>]
  
  VIEW-AS FILL-IN [NATIVE]
  [{SIZE-PIXELS | SIZE-CHARS | SIZE} width BY height]
  TOOLTIP <tooltip>]
  
  VIEW-AS RADIO-SET
  RADIO-BUTTONS label, value [,label, value] ...
  [HORIZONTAL [EXPAND] | VERTICAL]
  [{SIZE-PIXELS | SIZE-CHARS | SIZE} width BY height]
  [TOOLTIP <tooltip>]
  
  
  VIEW-AS SELECTION-LIST
  {{SIZE-PIXELS | SIZE-CHARS | SIZE} width BY height |
   INNER-CHARS num-chars INNER-LINES num-lines}}
  [SINGLE | MULTIPLE] [NO-DRAG] [SORT]
  [SCROLLBAR-HORIZONTAL]
  [SCROLLBAR-VERTICAL]
  [LIST-ITEMS value [,value] ... |
  
  VIEW-AS TEXT
  [{SIZE-PIXELS | SIZE-CHARS | SIZE} width BY height]
  [TOOLTIP <tooltip>]
  
  
  DEFINE {[[ NEW [ GLOBAL ]] SHARED ]| 
            [ PRIVATE | PROTECTED | PUBLIC ]
            [ STATIC ][ SERIALIZABLE | NON-SERIALIZABLE ]}
  VARIABLE variable-name
  {{   AS primitive-type-name 
       | AS [ CLASS ]{object-type-name}
       | LIKE field       }[ EXTENT [constant]]} 
  [ SERIALIZE-NAME serialize-name ]
  [ BGCOLOR expression]
  [ COLUMN-LABEL label]
  [ CONTEXT-HELP-ID expression]
  [ DCOLOR expression]
  [ DECIMALS n]
  [ DROP-TARGET ]
  [ FONT expression]
  [ FGCOLOR expression]
  [ FORMAT string]
  [ INITIAL
      { constant |{[ constant[ , constant]...]}}]
  [ LABEL string[ , string]...]
  [ MOUSE-POINTER expression]
  [ NO-UNDO ]
  [[ NOT ] CASE-SENSITIVE ]
  [ PFCOLOR expression]
  {[ view-as-phrase ]}
  {[ trigger-phrase ]}
  
  
  
  DEFINE [ PRIVATE ] RECTANGLE rectangle[ LIKE rectangle2]
  [ NO-FILL ]
  [{ EDGE-CHARS width}|{ EDGE-PIXELS width}] 
  [ DCOLOR expression]
  [ BGCOLOR expression]
  [ FGCOLOR expression]
  [ GRAPHIC-EDGE ]
  [ PFCOLOR expression]
  [ ROUNDED ]
  [ GROUP-BOX ]
  [size-phrase]
  [ TOOLTIP tooltip]
  {[trigger-phrase]}
  
  DEFINE {[[ NEW ] SHARED ]|[ PRIVATE ]}
  BROWSE browse-name
  QUERY query-name  
    [ SHARE-LOCK | EXCLUSIVE-LOCK | NO-LOCK ][ NO-WAIT ] 
  DISPLAY
    {column-list|record[ EXCEPT field...]} 
    [browse-enable-phrase] 
    {browse-options-phrase}
    [ CONTEXT-HELP-ID expression]
    [ DROP-TARGET ]
    [ TOOLTIP tooltip]
  """


class Reader:
	@property
	def txt(self):
		return self._txt
	
	@txt.setter
	def txt(self,ptxt):
		self._txt = ptxt
		self.txt_no_lines = self._txt.replace("\n"," ").lower()
		self._currentindex = 0
		self.start_index = 0
		self.len = len(self._txt)
		self.lenmo = self.len - 1
		self.index = 0

	@property
	def part(self):
		return self._txt[self.start_index:self.index]
	
	@property
	def current_line(self):
		index_start = self._txt.rfind("\n",0,self.index)
		index_stop = self._txt.find("\n",self.index)
		if index_start == -1: index_start = 0
		if index_stop == -1: index_stop = self.len
		return self._txt[index_start:index_stop]

	@property
	def current_line_index(self):
		return self._txt.count("\n",0,self.index)

	@property
	def index(self):
		return self._currentindex
	
	@index.setter
	def index(self,i):
		self._currentindex = i
		self.stopped = self._currentindex >= self.len
		if self.stopped:
			self.char = None
			return
		self.char = self._txt[i]
		
	def find(self,needle,index=None,alt=None):
		if index == None: index = self._currentindex
		i = self._txt.find(needle,index)
		return i if i != -1 else alt
		
	def char_before(self,index=None,alt=""):
		if index == None: index = self._currentindex
		return self._txt[index-1] if index > 0 else alt

	def char_after(self,index=None,alt=""):
		if index == None: index = self._currentindex
		return self._txt[index+1] if self.lenmo >= (index + 1) else alt
		
	


from Imports import *
"""

UNKNOWN                                            = 0
KEYWORD                                            = 1
STRING                                             = 2
NUMBER                                             = 3
TABLE                                              = 4
FIELD                                              = 5
VAR                                                = 6
BOOLEAN                                            = 7
EXPRESSION                                         = 8
PP_UNKNOWN                                         = 9
PP_SCOP                                            = 10
PP_GLOB                                            = 11
PP_UNDEF                                           = 12
PP_VAR                                             = 13
PP_PAR                                             = 14
PP_INCL                                            = 15
COMMAND                                            = 16

"""
from jivi.oe.ReaderParts.BigBlock import get as GetBigBlock
def check_ar(ar):
	good = 1
	for x in ar:
		if isinstance(x,list):
			if not check_ar(x): return 0
			continue
		if len(x) == 1: continue
		if x.endswith(":"): return 0
		if x.endswith("."): return 0
	return 1
	
	

def check_ar(ar):
	ret = []
	for a in ar:
		if not isinstance(a,list): continue
		if a[0] == PP_INCL and a[1][1]:
			ret.append(a[1][1])
	return ret
def old():

	for fp in Dir.files("_compiled",ex="json",full=1):
		a = File.jread(fp)
		
		if not check_ar(a):
			print(fp)
			exit()
				
	all = []
		
	for fp in Dir.files("_compiled",ex="json",full=1):
		a = File.jread(fp)
		all += check_ar(a)


	File.jwrite("all.json",all)
	
	
	
	
	
	
	
	
	
def do(rest):
	ar = GetBigBlock(rest)
	
	args = dict(ar=[],tab={})
	while rest:
		ar = rest.split(" ")
		first_word = ar[0]
		
		if first_word.startswith("&") and len(ar) > 2 and ar[1] == "=":
			rest = " ".join(ar[2:]).strip()
			if rest[0] in string_starts:
				
				index = rest.find(rest[0],1)
				args['tab'][first_word[1:].lower().strip()] = rest[1:index]
				rest = rest[index:]
			
				print(args)
				exit()
		else:
			args['ar'].append(first_word)
			rest = " ".join(ar[1:])
	
	
	


for fp in Dir.files("_compiled",ex="json",full=1):
	a = File.jread(fp)
	if not a: continue
	for x in a:
		if isinstance(x,int): continue
		if x[0] == PP_INCL and x[1][1]:
			print(x[1])
	
	


	
	
		
def create_pp_var_concat(ar):
	new_ar = []
	ret_ar = [PP_VAR_CONCAT,new_ar]
	s = ar[0]
	
	
	index = s.find("{&")
	while index != -1:
		if index > 0:
			new_ar.append([STRING,s[:index],NONE])
		end_index = s.find("}",index+2)
		if end_index == -1:
			print("OEIEIEIEIEIE!!!")
			print(s)
			print(ar)
			exit()
		x = s[index+2:end_index].lower().strip()
		new_ar.append([PP_VAR,x])
		s = s[end_index+1:]
		index = s.find("{&")
		
	if s:
		new_ar.append([STRING,[s,UNKNOWN]])
	return ret_ar
		
		
		
def create_string_concatx(ar):
	new_ar = []
	ret_ar = [STRING_CONCAT,new_ar]
	s = ar[0]
	
	
	index = s.find("{&")
	while index != -1:
		if index > 0:
			new_ar.append([STRING,s[:index],NONE])
		end_index = s.find("}",index+2)
		if end_index == -1:
			print("OEIEIEIEIEIE!!!")
			print(s)
			print(ar)
			exit()
		x = s[index+2:end_index].lower().strip()
		new_ar.append([PP_VAR,x])
		s = s[end_index+1:]
		index = s.find("{&")
		
	if s:
		new_ar.append([STRING,s])


	return ret_ar
		
	
	
	
def do_ppc_read_condition(ar):
	count = 0
	for i,a in enumerate(ar):
		if not isinstance(ar[i],list): continue
		x = ar[i]
		if x[0] == PP_CONDITION:
			if x[1] == pp_if:
				count += 1
				continue
			if x[1] == pp_then:
				if not count:
					return i
				continue
			if x[1] == pp_endif:
				count -= 1
				continue
	return None
	

def do_ppc_read_block(ar):
	count = 0
	for i,a in enumerate(ar):
		if not isinstance(ar[i],list): continue
		x = ar[i]
		if x[0] == PP_CONDITION:
			if x[1] == pp_if:
				count += 1
				continue
			if x[1] == pp_elseif or x[1] == pp_else:
				if not count: return i
				continue
			if x[1] == pp_endif:
				if not count: return i
				count -= 1
				continue
	if not count:
		return len(ar)
	
	return None

def do_ppc(ar):
	ori = ar[:]
	ar = ar[1:-1]
	
	condition_statements = []
	code_blocks = []
	ret_ar = [PP_CONDITIONBLOCK,[condition_statements,code_blocks]]
	
	index = do_ppc_read_condition(ar)
	if not index:
		print("do_ppc failed !" + str(ar))
		exit()

	condition_statements.append(ar[:index])
	
	ar = ar[index+1:]
	index = do_ppc_read_block(ar)
	if index is None :
		print("do_ppc do_ppc_read_block failed !" + str(ar))
		exit()
	
	code_block = ar[:index]
	code_blocks.append(code_block)
	ar = ar[index+1:]
	if ar:
		print(make_readable_ar(ar))
		print(make_readable_ar(ori))
		exit()
	return ret_ar
def do_ppc_find_stop(ar,start=0):
	started = 0
	count = 0
	for i in range(start,len(ar)):
		x = ar[i]
		if isinstance(x,list) and x[0] == PP_CONDITION:
			if x[1] == "&if":
				count +=1
			elif x[1] == "&endif":
				count -= 1
			if count and (not started): started = 1
			if started and (not count):
				return i + 1
				



KEYWORD_AR = [
 "&else",
 "&elseif",
 "&endif",
 "&global-define",
 "&if",
 "&message",
 "&scoped-define",
 "&then",
 "&undefine",
 "&webstream",
 "'",
 "*",
 "+",
 "-",
 ".",
 "/",
 ":",
 "<",
 "<=",
 "<>",
 "=",
 ">",
 ">=",
 "?",
 "@",
 "[",
 "]",
 "^",
 "absolute",
 "abstract",
 "accelerator",
 "accumulate",
 "active-form",
 "active-window",
 "add",
 "add-buffer",
 "add-calc-column",
 "add-columns-from",
 "add-events-procedure",
 "add-fields-from",
 "add-first",
 "add-index-field",
 "add-last",
 "add-like-column",
 "add-like-field",
 "add-like-index",
 "add-new-field",
 "add-new-index",
 "add-schema-location",
 "add-super-procedure",
 "adm-data",
 "advise",
 "alert-box",
 "alias",
 "all",
 "allow-column-searching",
 "allow-replication",
 "alter",
 "always-on-top",
 "ambiguous",
 "analyze",
 "and",
 "ansi-only",
 "any",
 "anywhere",
 "append",
 "appl-alert-boxes",
 "appl-context-id",
 "application",
 "apply",
 "appserver-info",
 "appserver-password",
 "appserver-userid",
 "array-message",
 "as",
 "ascending",
 "ask-overwrite",
 "assembly",
 "assign",
 "async-request-count",
 "async-request-handle",
 "asynchronous",
 "at",
 "attached-pairlist",
 "attr-space",
 "audit-control",
 "audit-enabled",
 "audit-event-context",
 "audit-policy",
 "authentication-failed",
 "authorization",
 "auto-completion",
 "auto-end-key",
 "auto-endkey",
 "auto-go",
 "auto-indent",
 "auto-resize",
 "auto-return",
 "auto-synchronize",
 "auto-zap",
 "automatic",
 "available",
 "available-formats",
 "average",
 "avg",
 "background",
 "backwards",
 "base-ade",
 "base-key",
 "base64-decode",
 "base64-encode",
 "batch-mode",
 "batch-size",
 "before-hide",
 "begin-event-group",
 "begins",
 "bell",
 "between",
 "bgcolor",
 "big-endian",
 "binary",
 "bind",
 "bind-where",
 "blank",
 "block-iteration-display",
 "block-level",
 "border-bottom-chars",
 "border-bottom-pixels",
 "border-left-chars",
 "border-left-pixels",
 "border-right-chars",
 "border-right-pixels",
 "border-top-chars",
 "border-top-pixels",
 "box",
 "box-selectable",
 "break",
 "browse",
 "buffer",
 "buffer-chars",
 "buffer-compare",
 "buffer-copy",
 "buffer-create",
 "buffer-delete",
 "buffer-field",
 "buffer-handle",
 "buffer-lines",
 "buffer-name",
 "buffer-partition-id",
 "buffer-release",
 "buffer-value",
 "button",
 "by",
 "by-pointer",
 "by-variant-pointer",
 "cache",
 "cache-size",
 "call",
 "call-name",
 "call-type",
 "can-create",
 "can-delete",
 "can-do",
 "can-do-domain-support",
 "can-find",
 "can-query",
 "can-read",
 "can-set",
 "can-write",
 "cancel-break",
 "cancel-button",
 "caps",
 "careful-paint",
 "case",
 "case-sensitive",
 "cast",
 "catch",
 "cdecl",
 "centered",
 "chained",
 "character",
 "character_length",
 "charset",
 "check",
 "checked",
 "choose",
 "chr",
 "class",
 "class-type",
 "clear",
 "clear-appl-context",
 "clear-log",
 "clear-selection",
 "clear-sort-arrows",
 "client-connection-id",
 "client-principal",
 "client-tty",
 "client-type",
 "client-workstation",
 "clipboard",
 "close",
 "close-log",
 "code",
 "codebase-locator",
 "codepage",
 "codepage-convert",
 "col-of",
 "collate",
 "colon",
 "colon-aligned",
 "color",
 "color-table",
 "column",
 "column-bgcolor",
 "column-dcolor",
 "column-fgcolor",
 "column-font",
 "column-label",
 "column-movable",
 "column-of",
 "column-pfcolor",
 "column-read-only",
 "column-resizable",
 "column-scrolling",
 "columns",
 "com-handle",
 "com-self",
 "combo-box",
 "command",
 "compares",
 "compile",
 "compiler",
 "complete",
 "config-name",
 "connect",
 "connected",
 "constructor",
 "contains",
 "contents",
 "context",
 "context-help",
 "context-help-file",
 "context-help-id",
 "context-popup",
 "control",
 "control-box",
 "control-frame",
 "convert",
 "convert-3d-colors",
 "convert-to-offset",
 "copy-dataset",
 "copy-lob",
 "copy-sax-attributes",
 "copy-temp-table",
 "count",
 "count-of",
 "cpcase",
 "cpcoll",
 "cpinternal",
 "cplog",
 "cpprint",
 "cprcodein",
 "cprcodeout",
 "cpstream",
 "cpterm",
 "crc-value",
 "create",
 "create-like",
 "create-like-sequential",
 "create-node-namespace",
 "create-result-list-entry",
 "create-test-file",
 "current",
 "current-changed",
 "current-column",
 "current-environment",
 "current-iteration",
 "current-language",
 "current-query",
 "current-request-info",
 "current-response-info",
 "current-result-row",
 "current-row-modified",
 "current-value",
 "current-window",
 "current_date",
 "cursor",
 "cursor-char",
 "cursor-line",
 "cursor-offset",
 "data-bind",
 "data-entry-return",
 "data-relation",
 "data-source",
 "data-source-complete-map",
 "data-source-modified",
 "data-source-rowid",
 "data-type",
 "database",
 "dataservers",
 "dataset",
 "dataset-handle",
 "date",
 "date-format",
 "datetime",
 "datetime-tz",
 "day",
 "db-context",
 "db-references",
 "dbcodepage",
 "dbcollation",
 "dbname",
 "dbparam",
 "dbrestrictions",
 "dbtaskid",
 "dbtype",
 "dbversion",
 "dcolor",
 "dde",
 "dde-error",
 "dde-id",
 "dde-item",
 "dde-name",
 "dde-topic",
 "deblank",
 "debug",
 "debug-alert",
 "debug-list",
 "debugger",
 "decimal",
 "decimals",
 "declare",
 "declare-namespace",
 "decrypt",
 "default",
 "default-buffer-handle",
 "default-button",
 "default-commit",
 "default-extension",
 "default-noxlate",
 "default-value",
 "default-window",
 "define",
 "define-user-event-manager",
 "defined",
 "delegate",
 "delete",
 "delete procedure",
 "delete-char",
 "delete-current-row",
 "delete-line",
 "delete-result-list-entry",
 "delete-selected-row",
 "delete-selected-rows",
 "delimiter",
 "descending",
 "deselect-focused-row",
 "deselect-rows",
 "deselect-selected-row",
 "deselection",
 "destructor",
 "dialog-box",
 "dictionary",
 "dir",
 "disable",
 "disable-auto-zap",
 "disable-dump-triggers",
 "disable-load-triggers",
 "disabled",
 "disconnect",
 "display",
 "display-message",
 "display-type",
 "distinct",
 "do",
 "domain-description",
 "domain-name",
 "domain-type",
 "dos",
 "double",
 "down",
 "drag-enabled",
 "drop",
 "drop-down",
 "drop-down-list",
 "drop-file-notify",
 "drop-target",
 "ds-close-cursor",
 "dslog-manager",
 "dump",
 "dynamic",
 "dynamic-enum",
 "dynamic-function",
 "dynamic-invoke",
 "each",
 "echo",
 "edge-chars",
 "edge-pixels",
 "edit-can-paste",
 "edit-can-undo",
 "edit-clear",
 "edit-copy",
 "edit-cut",
 "edit-paste",
 "edit-undo",
 "editing",
 "editor",
 "else",
 "empty",
 "empty-temp-table",
 "enable",
 "enabled-fields",
 "encode",
 "encrypt",
 "encrypt-audit-mac-key",
 "encryption-salt",
 "end",
 "end-document",
 "end-element",
 "end-event-group",
 "end-file-drop",
 "end-key",
 "end-move",
 "end-resize",
 "end-row-resize",
 "end-user-prompt",
 "endkey",
 "entered",
 "entity-expansion-limit",
 "entry",
 "enum",
 "error",
 "error-column",
 "error-row",
 "error-stack-trace",
 "error-status",
 "escape",
 "etime",
 "event",
 "event-group-id",
 "event-procedure",
 "event-procedure-context",
 "event-type",
 "events",
 "except",
 "exclusive-id",
 "exclusive-lock",
 "exclusive-web-user",
 "execute",
 "exists",
 "exp",
 "expand",
 "expandable",
 "explicit",
 "export",
 "export-principal",
 "extended",
 "extent",
 "external",
 "false",
 "fetch",
 "fetch-selected-row",
 "fgcolor",
 "fields",
 "file",
 "file-create-date",
 "file-create-time",
 "file-information",
 "file-mod-date",
 "file-mod-time",
 "file-name",
 "file-offset",
 "file-size",
 "file-type",
 "filename",
 "fill",
 "fill-in",
 "filled",
 "filters",
 "final",
 "finally",
 "find",
 "find-by-rowid",
 "find-case-sensitive",
 "find-current",
 "find-first",
 "find-global",
 "find-last",
 "find-next-occurrence",
 "find-prev-occurrence",
 "find-select",
 "find-unique",
 "find-wrap-around",
 "finder",
 "first",
 "first-asynch-request",
 "first-child",
 "first-column",
 "first-form",
 "first-object",
 "first-of",
 "first-procedure",
 "first-server",
 "first-tab-item",
 "fit-last-column",
 "fixed-only",
 "flat-button",
 "float",
 "focus",
 "focused-row",
 "focused-row-selected",
 "font",
 "font-table",
 "for",
 "force-file",
 "foreground",
 "form input",
 "form-long-input",
 "format",
 "formatted",
 "forwards",
 "fragment",
 "frame",
 "frame-col",
 "frame-db",
 "frame-down",
 "frame-field",
 "frame-file",
 "frame-index",
 "frame-line",
 "frame-name",
 "frame-row",
 "frame-spacing",
 "frame-value",
 "frame-x",
 "frame-y",
 "frequency",
 "from",
 "from-chars",
 "from-current",
 "from-pixels",
 "full-height-chars",
 "full-height-pixels",
 "full-pathname",
 "full-width-chars",
 "full-width-pixels",
 "function",
 "function-call-type",
 "gateways",
 "generate-md5",
 "generate-pbe-key",
 "generate-pbe-salt",
 "generate-random-key",
 "generate-uuid",
 "get",
 "get-attr-call-type",
 "get-attribute-node",
 "get-binary-data",
 "get-blue-value",
 "get-browse-column",
 "get-buffer-handle",
 "get-byte",
 "get-callback-proc-context",
 "get-callback-proc-name",
 "get-cgi-list",
 "get-cgi-long-value",
 "get-cgi-value",
 "get-class",
 "get-codepages",
 "get-collations",
 "get-config-value",
 "get-current",
 "get-double",
 "get-dropped-file",
 "get-dynamic",
 "get-error-column",
 "get-error-row",
 "get-file",
 "get-file-name",
 "get-file-offset",
 "get-first",
 "get-float",
 "get-green-value",
 "get-index-by-namespace-name",
 "get-index-by-qname",
 "get-int64",
 "get-iteration",
 "get-key-value",
 "get-last",
 "get-localname-by-index",
 "get-long",
 "get-message",
 "get-next",
 "get-number",
 "get-pointer-value",
 "get-prev",
 "get-printers",
 "get-property",
 "get-qname-by-index",
 "get-red-value",
 "get-repositioned-row",
 "get-rgb-value",
 "get-selected-widget",
 "get-short",
 "get-signature",
 "get-size",
 "get-string",
 "get-tab-item",
 "get-text-height-chars",
 "get-text-height-pixels",
 "get-text-width-chars",
 "get-text-width-pixels",
 "get-type-by-index",
 "get-type-by-namespace-name",
 "get-type-by-qname",
 "get-unsigned-long",
 "get-unsigned-short",
 "get-uri-by-index",
 "get-value-by-index",
 "get-value-by-namespace-name",
 "get-value-by-qname",
 "get-wait-state",
 "getbyte",
 "global",
 "go-on",
 "go-pending",
 "grant",
 "graphic-edge",
 "grid-factor-horizontal",
 "grid-factor-vertical",
 "grid-snap",
 "grid-unit-height-chars",
 "grid-unit-height-pixels",
 "grid-unit-width-chars",
 "grid-unit-width-pixels",
 "grid-visible",
 "group",
 "guid",
 "handle",
 "handler",
 "has-records",
 "having",
 "header",
 "height-chars",
 "height-pixels",
 "help",
 "hex-decode",
 "hex-encode",
 "hidden",
 "hide",
 "horizontal",
 "host-byte-order",
 "html-charset",
 "html-end-of-line",
 "html-end-of-page",
 "html-frame-begin",
 "html-frame-end",
 "html-header-begin",
 "html-header-end",
 "html-title-begin",
 "html-title-end",
 "hwnd",
 "icon",
 "if",
 "image",
 "image-down",
 "image-insensitive",
 "image-size",
 "image-size-chars",
 "image-size-pixels",
 "image-up",
 "immediate-display",
 "implements",
 "import",
 "import-principal",
 "in",
 "in-handle",
 "increment-exclusive-id",
 "index",
 "index-hint",
 "index-information",
 "indexed-reposition",
 "indicator",
 "information",
 "inherit-bgcolor",
 "inherit-fgcolor",
 "inherits",
 "initial",
 "initial-dir",
 "initial-filter",
 "initialize-document-type",
 "initiate",
 "inner-chars",
 "inner-lines",
 "input",
 "input-output",
 "input-value",
 "insert",
 "insert-attribute",
 "insert-backtab",
 "insert-file",
 "insert-row",
 "insert-string",
 "insert-tab",
 "int64",
 "integer",
 "interface",
 "internal-entries",
 "into",
 "invoke",
 "is",
 "is-attr-space",
 "is-class",
 "is-json",
 "is-lead-byte",
 "is-open",
 "is-parameter-set",
 "is-partitioned",
 "is-row-selected",
 "is-selected",
 "is-xml",
 "item",
 "items-per-row",
 "join",
 "join-by-sqldb",
 "kblabel",
 "keep-connection-open",
 "keep-frame-z-order",
 "keep-messages",
 "keep-security-cache",
 "keep-tab-order",
 "key",
 "key-code",
 "key-function",
 "key-label",
 "keycode",
 "keyfunction",
 "keylabel",
 "keys",
 "keywords",
 "keywords-all",
 "label",
 "label-bgcolor",
 "label-dcolor",
 "label-fgcolor",
 "label-font",
 "label-pfcolor",
 "labels",
 "labels-have-colons",
 "landscape",
 "languages",
 "large",
 "large-to-small",
 "last",
 "last-asynch-request",
 "last-batch",
 "last-child",
 "last-event",
 "last-form",
 "last-key",
 "last-object",
 "last-of",
 "last-procedure",
 "last-server",
 "last-tab-item",
 "lastkey",
 "lc",
 "ldbname",
 "leave",
 "left-aligned",
 "left-trim",
 "length",
 "library",
 "like",
 "like-sequential",
 "line",
 "line-counter",
 "list-events",
 "list-item-pairs",
 "list-items",
 "list-property-names",
 "list-query-attrs",
 "list-set-attrs",
 "list-widgets",
 "listing",
 "literal-question",
 "little-endian",
 "load",
 "load-domains",
 "load-icon",
 "load-image",
 "load-image-down",
 "load-image-insensitive",
 "load-image-up",
 "load-mouse-pointer",
 "load-picture",
 "load-small-icon",
 "local-name",
 "local-version-info",
 "locator-column-number",
 "locator-line-number",
 "locator-public-id",
 "locator-system-id",
 "locator-type",
 "lock-registration",
 "locked",
 "log-audit-event",
 "log-manager",
 "logical",
 "login-expiration-timestamp",
 "login-host",
 "login-state",
 "logout",
 "longchar",
 "lookahead",
 "lookup",
 "machine-class",
 "mandatory",
 "manual-highlight",
 "map",
 "margin-extra",
 "margin-height-chars",
 "margin-height-pixels",
 "margin-width-chars",
 "margin-width-pixels",
 "mark-new",
 "mark-row-state",
 "matches",
 "max-button",
 "max-chars",
 "max-data-guess",
 "max-height",
 "max-height-chars",
 "max-height-pixels",
 "max-rows",
 "max-size",
 "max-value",
 "max-width-chars",
 "max-width-pixels",
 "maximize",
 "maximum",
 "maximum-level",
 "md5-digest",
 "member",
 "memptr",
 "memptr-to-node-value",
 "menu",
 "menu-bar",
 "menu-item",
 "menu-key",
 "menu-mouse",
 "menubar",
 "merge-by-field",
 "message",
 "message-area",
 "message-area-font",
 "message-lines",
 "method",
 "min-button",
 "min-column-width-chars",
 "min-column-width-pixels",
 "min-height-chars",
 "min-height-pixels",
 "min-size",
 "min-value",
 "min-width-chars",
 "min-width-pixels",
 "minimum",
 "modified",
 "modulo",
 "month",
 "mouse",
 "mouse-pointer",
 "movable",
 "move-after-tab-item",
 "move-before-tab-item",
 "move-column",
 "move-to-bottom",
 "move-to-eof",
 "move-to-top",
 "mpe",
 "mtime",
 "multi-compile",
 "multiple",
 "multiple-key",
 "multitasking-interval",
 "must-exist",
 "name",
 "namespace-prefix",
 "namespace-uri",
 "native",
 "needs-appserver-prompt",
 "needs-prompt",
 "new",
 "new-instance",
 "new-row",
 "next",
 "next-column",
 "next-prompt",
 "next-rowid",
 "next-sibling",
 "next-tab-item",
 "next-value",
 "no",
 "no-apply",
 "no-array-message",
 "no-assign",
 "no-attr-list",
 "no-attr-space",
 "no-auto-validate",
 "no-bind-where",
 "no-box",
 "no-console",
 "no-convert",
 "no-convert-3d-colors",
 "no-current-value",
 "no-debug",
 "no-drag",
 "no-echo",
 "no-empty-space",
 "no-error",
 "no-fill",
 "no-focus",
 "no-help",
 "no-hide",
 "no-index-hint",
 "no-inherit-bgcolor",
 "no-inherit-fgcolor",
 "no-join-by-sqldb",
 "no-labels",
 "no-lobs",
 "no-lock",
 "no-lookahead",
 "no-map",
 "no-message",
 "no-pause",
 "no-prefetch",
 "no-row-markers",
 "no-scrollbar-vertical",
 "no-separate-connection",
 "no-separators",
 "no-tab-stop",
 "no-underline",
 "no-undo",
 "no-validate",
 "no-wait",
 "no-word-wrap",
 "node-value-to-memptr",
 "nonamespace-schema-location",
 "none",
 "normalize",
 "not",
 "not-active",
 "now",
 "null",
 "num-aliases",
 "num-buffers",
 "num-buttons",
 "num-columns",
 "num-copies",
 "num-dbs",
 "num-dropped-files",
 "num-entries",
 "num-fields",
 "num-formats",
 "num-items",
 "num-iterations",
 "num-lines",
 "num-locked-columns",
 "num-messages",
 "num-parameters",
 "num-references",
 "num-replaced",
 "num-results",
 "num-selected-rows",
 "num-selected-widgets",
 "num-tabs",
 "num-to-retain",
 "num-visible-columns",
 "numeric",
 "numeric-format",
 "octet-length",
 "of",
 "off",
 "ok",
 "ok-cancel",
 "old",
 "on",
 "on-frame-border",
 "open",
 "opsys",
 "option",
 "or",
 "ordered-join",
 "ordinal",
 "os-append",
 "os-command",
 "os-copy",
 "os-create-dir",
 "os-delete",
 "os-dir",
 "os-drives",
 "os-error",
 "os-getenv",
 "os-rename",
 "otherwise",
 "output",
 "overlay",
 "override",
 "owner",
 "page",
 "page-bottom",
 "page-number",
 "page-size",
 "page-top",
 "page-width",
 "paged",
 "parameter",
 "parent",
 "parse-status",
 "partial-key",
 "pascal",
 "password-field",
 "pathname",
 "pause",
 "pbe-hash-algorithm",
 "pbe-key-rounds",
 "pdbname",
 "persistent",
 "persistent-cache-disabled",
 "pfcolor",
 "pixels",
 "pixels-per-column",
 "pixels-per-row",
 "popup-menu",
 "popup-only",
 "portrait",
 "position",
 "precision",
 "prefer-dataset",
 "prepare-string",
 "prepared",
 "preprocess",
 "preselect",
 "prev",
 "prev-column",
 "prev-sibling",
 "prev-tab-item",
 "primary",
 "printer",
 "printer-control-handle",
 "printer-hdc",
 "printer-name",
 "printer-port",
 "printer-setup",
 "private",
 "private-data",
 "privileges",
 "proc-handle",
 "proc-status",
 "proc-text",
 "proc-text-buffer",
 "procedure",
 "procedure-call-type",
 "procedure-type",
 "process",
 "profiler",
 "program-name",
 "progress",
 "progress-source",
 "prompt",
 "prompt-for",
 "promsgs",
 "propath",
 "property",
 "protected",
 "proversion",
 "proxy",
 "proxy-password",
 "proxy-userid",
 "public",
 "public-id",
 "publish",
 "published-events",
 "put",
 "put-byte",
 "put-double",
 "put-float",
 "put-int64",
 "put-key-value",
 "put-long",
 "put-short",
 "put-string",
 "put-unsigned-long",
 "putbyte",
 "query",
 "query-close",
 "query-off-end",
 "query-open",
 "query-prepare",
 "query-tuning",
 "question",
 "quit",
 "quoter",
 "r-index",
 "radio-buttons",
 "radio-set",
 "random",
 "raw",
 "raw-transfer",
 "rcode-information",
 "read-available",
 "read-exact-num",
 "read-file",
 "read-json",
 "read-only",
 "read-xml",
 "readkey",
 "real",
 "recid",
 "record-length",
 "rectangle",
 "recursive",
 "reference-only",
 "refresh",
 "refresh-audit-policy",
 "refreshable",
 "register-domain",
 "release",
 "remote",
 "remove-events-procedure",
 "remove-super-procedure",
 "repeat",
 "replace",
 "replace-selection-text",
 "reposition",
 "reposition-backward",
 "reposition-forward",
 "reposition-mode",
 "reposition-to-row",
 "reposition-to-rowid",
 "request",
 "request-info",
 "reset",
 "resizable",
 "resize",
 "response-info",
 "restart-row",
 "restart-rowid",
 "retain",
 "retain-shape",
 "retry",
 "retry-cancel",
 "return",
 "return-inserted",
 "return-to-start-dir",
 "return-value",
 "return-value-data-type",
 "returns",
 "reverse-from",
 "revert",
 "revoke",
 "rgb-value",
 "right-aligned",
 "right-trim",
 "roles",
 "round",
 "routine-level",
 "row",
 "row-height-chars",
 "row-height-pixels",
 "row-markers",
 "row-of",
 "row-resizable",
 "rowid",
 "rule",
 "run",
 "run-procedure",
 "save",
 "save cache",
 "save-as",
 "save-file",
 "sax-complete",
 "sax-parse",
 "sax-parse-first",
 "sax-parse-next",
 "sax-parser-error",
 "sax-running",
 "sax-uninitialized",
 "sax-write-begin",
 "sax-write-complete",
 "sax-write-content",
 "sax-write-element",
 "sax-write-error",
 "sax-write-idle",
 "sax-write-tag",
 "sax-writer",
 "schema",
 "schema-location",
 "schema-marshal",
 "schema-path",
 "screen",
 "screen-io",
 "screen-lines",
 "screen-value",
 "scroll",
 "scroll-bars",
 "scroll-delta",
 "scroll-offset",
 "scroll-to-current-row",
 "scroll-to-item",
 "scroll-to-selected-row",
 "scrollable",
 "scrollbar-horizontal",
 "scrollbar-vertical",
 "scrolled-row-position",
 "scrolling",
 "sdbname",
 "seal",
 "seal-timestamp",
 "search",
 "search-self",
 "search-targer",
 "section",
 "security-policy",
 "seek",
 "select",
 "select-all",
 "select-focused-row",
 "select-next-row",
 "select-prev-row",
 "select-row",
 "selectable",
 "selected",
 "selection",
 "selection-end",
 "selection-list",
 "selection-start",
 "selection-text",
 "self",
 "send",
 "send-sql-statement",
 "sensitive",
 "separate-connection",
 "separator-fgcolor",
 "separators",
 "serializable",
 "serialize-hidden",
 "serialize-name",
 "server",
 "server-connection-bound",
 "server-connection-bound-request",
 "server-connection-context",
 "server-connection-id",
 "server-operating-mode",
 "session",
 "session-id",
 "set",
 "set-appl-context",
 "set-attr-call-type",
 "set-attribute-node",
 "set-blue-value",
 "set-break",
 "set-buffers",
 "set-callback",
 "set-client",
 "set-commit",
 "set-contents",
 "set-current-value",
 "set-db-client",
 "set-dynamic",
 "set-event-manager-option",
 "set-green-value",
 "set-input-source",
 "set-option",
 "set-output-destination",
 "set-parameter",
 "set-pointer-value",
 "set-property",
 "set-red-value",
 "set-repositioned-row",
 "set-rgb-value",
 "set-rollback",
 "set-selection",
 "set-size",
 "set-sort-arrow",
 "set-wait-state",
 "setuserid",
 "sha1-digest",
 "share-lock",
 "shared",
 "show-in-taskbar",
 "show-stats",
 "side-label-handle",
 "side-labels",
 "signature",
 "silent",
 "simple",
 "single",
 "single-run",
 "singleton",
 "size",
 "size-chars",
 "size-pixels",
 "skip",
 "skip-deleted-record",
 "slider",
 "small-icon",
 "small-title",
 "smallint",
 "some",
 "sort",
 "sort-ascending",
 "sort-number",
 "source",
 "source-procedure",
 "space",
 "sql",
 "sqrt",
 "ssl-server-name",
 "standalone",
 "start",
 "start-document",
 "start-element",
 "start-move",
 "start-resize",
 "start-row-resize",
 "state-detail",
 "static",
 "status",
 "status-area",
 "status-area-font",
 "stdcall",
 "stop",
 "stop-after",
 "stop-parsing",
 "stopped",
 "stored-procedure",
 "stream",
 "stream-handle",
 "stream-io",
 "stretch-to-fit",
 "strict",
 "strict-entity-resolution",
 "string",
 "string-value",
 "string-xref",
 "sub-average",
 "sub-count",
 "sub-maximum",
 "sub-menu",
 "sub-minimum",
 "sub-total",
 "subscribe",
 "substitute",
 "substring",
 "subtype",
 "sum",
 "super",
 "super-procedures",
 "suppress-namespace-processing",
 "suppress-warnings",
 "symmetric-encryption-algorithm",
 "symmetric-encryption-iv",
 "symmetric-encryption-key",
 "symmetric-support",
 "system-alert-boxes",
 "system-dialog",
 "system-help",
 "system-id",
 "tab-position",
 "tab-stop",
 "table",
 "table-handle",
 "table-number",
 "table-scan",
 "target",
 "target-procedure",
 "temp-directory",
 "temp-table",
 "temp-table-prepare",
 "terminal",
 "terminate",
 "text",
 "text-cursor",
 "text-seg-grow",
 "text-selected",
 "then",
 "this-object",
 "this-procedure",
 "thread-safe",
 "three-d",
 "through",
 "throw",
 "thru",
 "tic-marks",
 "time",
 "time-source",
 "title",
 "title-bgcolor",
 "title-dcolor",
 "title-fgcolor",
 "title-font",
 "to",
 "to-rowid",
 "today",
 "toggle-box",
 "tooltip",
 "tooltips",
 "top-nav-query",
 "top-only",
 "topic",
 "total",
 "trailing",
 "trans",
 "trans-init-procedure",
 "transaction",
 "transaction-mode",
 "transparent",
 "trigger",
 "triggers",
 "trim",
 "true",
 "truncate",
 "type",
 "type-of",
 "unbox",
 "unbuffered",
 "underline",
 "undo",
 "unformatted",
 "union",
 "unique",
 "unique-id",
 "unique-match",
 "unix",
 "unless-hidden",
 "unload",
 "unsigned-long",
 "unsubscribe",
 "up",
 "update",
 "update-attribute",
 "url",
 "url-decode",
 "url-encode",
 "url-password",
 "url-userid",
 "use",
 "use-dict-exps",
 "use-filename",
 "use-index",
 "use-revvideo",
 "use-text",
 "use-underline",
 "use-widget-pool",
 "user",
 "user-id",
 "userid",
 "using",
 "v6display",
 "v6frame",
 "valid-event",
 "valid-handle",
 "valid-object",
 "validate",
 "validate-expression",
 "validate-message",
 "validate-seal",
 "validation-enabled",
 "value",
 "value-changed",
 "values",
 "variable",
 "verbose",
 "version",
 "vertical",
 "view",
 "view-as",
 "view-first-column-on-reopen",
 "virtual-height-chars",
 "virtual-height-pixels",
 "virtual-width-chars",
 "virtual-width-pixels",
 "visible",
 "void",
 "wait",
 "wait-for",
 "warning",
 "web-context",
 "weekday",
 "when",
 "where",
 "while",
 "widget",
 "widget-enter",
 "widget-id",
 "widget-leave",
 "widget-pool",
 "width-chars",
 "width-pixels",
 "window",
 "window-maximized",
 "window-minimized",
 "window-name",
 "window-normal",
 "window-state",
 "window-system",
 "with",
 "word-index",
 "word-wrap",
 "work-area-height-pixels",
 "work-area-width-pixels",
 "work-area-x",
 "work-area-y",
 "work-table",
 "workfile",
 "write",
 "write-cdata",
 "write-characters",
 "write-comment",
 "write-data-element",
 "write-empty-element",
 "write-entity-ref",
 "write-external-dtd",
 "write-fragment",
 "write-json",
 "write-message",
 "write-processing-instruction",
 "write-status",
 "write-xml",
 "write-xmlschema",
 "x",
 "x-of",
 "xcode",
 "xml-data-type",
 "xml-entity-expansion-limit",
 "xml-node-type",
 "xml-schema-path",
 "xml-strict-entity-resolution",
 "xml-suppress-namespace-processing",
 "xref",
 "xref-xml",
 "y",
 "y-of",
 "year",
 "year-offset",
 "yes",
 "yes-no",
 "yes-no-cancel",
 "{&batch-mode}",
 "{&file-name}",
 "{&line-number}",
 "{&opsys}",
 "{&process-architecture}",
 "{&sequence}",
 "{&window-system}"
]
KEYWORD_TAB = {
 "browse": 137,
 "buffer": 138,
 "button": 151,
 "buttons": 151,
 "data-source": 293,
 "dataset": 300,
 "enum": 430,
 "event": 438,
 "frame": 522,
 "image": 664,
 "menu": 846,
 "parameter": 1017,
 "property": 1074,
 "query": 1095,
 "rectangle": 1121,
 "stream": 1338,
 "sub-menu": 1350,
 "temp-table": 1379,
 "variable": 1473,
 "work-table": 1515,
 "workfile": 1516,
 "fram": 522,
 "param": 1017,
 "parame": 1017,
 "paramet": 1017,
 "paramete": 1017,
 "rect": 1121,
 "recta": 1121,
 "rectan": 1121,
 "rectang": 1121,
 "rectangl": 1121,
 "sub-": 1350,
 "sub-m": 1350,
 "sub-me": 1350,
 "sub-men": 1350,
 "var": 1473,
 "vari": 1473,
 "varia": 1473,
 "variab": 1473,
 "variabl": 1473,
 "work-tab": 1515,
 "work-tabl": 1515,
 "character": 180,
 "char": 180,
 "chara": 180,
 "charac": 180,
 "charact": 180,
 "characte": 180,
 "com-handle": 225,
 "date": 302,
 "datetime": 304,
 "datetime-tz": 305,
 "decimal": 329,
 "dec": 329,
 "deci": 329,
 "decim": 329,
 "decima": 329,
 "handle": 638,
 "int64": 704,
 "integer": 705,
 "int": 705,
 "inte": 705,
 "integ": 705,
 "intege": 705,
 "logical": 808,
 "longchar": 813,
 "memptr": 844,
 "raw": 1108,
 "recid": 1119,
 "rowid": 1174,
 "&glob": 3,
 "&scop": 6,
 "&undef": 8,
 "{&batch}": 1550,
 "{&line-numbe}": 1552,
 "{&window-sys}": 1556,
 "abs": 28,
 "accum": 31,
 "ambig": 59,
 "analyz": 60,
 "appl-alert": 66,
 "asc": 75,
 "attr": 84,
 "auto-comp": 91,
 "auto-ind": 95,
 "auto-ret": 97,
 "auto-z": 99,
 "avail": 101,
 "ave": 103,
 "back": 105,
 "backward": 106,
 "batch": 111,
 "before-h": 113,
 "bgc": 118,
 "block-lev": 125,
 "border-b": 126,
 "border-bottom-p": 127,
 "border-l": 128,
 "border-left-p": 129,
 "border-r": 130,
 "border-right-p": 131,
 "border-t": 132,
 "border-top-p": 133,
 "box-select": 135,
 "case-sen": 174,
 "center": 178,
 "clear-select": 192,
 "clear-sort-arrow": 193,
 "colon-align": 209,
 "col": 212,
 "column-lab": 217,
 "convert-to-offs": 249,
 "current-env": 275,
 "current-lang": 277,
 "curs": 286,
 "data-entry-ret": 291,
 "data-rel": 292,
 "data-t": 297,
 "date-f": 303,
 "dbrest": 313,
 "dbvers": 316,
 "dde-i": 320,
 "debu": 325,
 "defaut-b": 336,
 "default-ex": 338,
 "default-noxl": 339,
 "def": 342,
 "del": 346,
 "desc": 355,
 "dict": 362,
 "discon": 369,
 "disp": 370,
 "display-t": 372,
 "edge": 396,
 "edge-p": 397,
 "error-col": 432,
 "error-stat": 435,
 "event-t": 442,
 "exclusive": 446,
 "fgc": 462,
 "field": 463,
 "file-info": 467,
 "file-off": 471,
 "first-proc": 501,
 "first-tab-i": 503,
 "fore": 515,
 "form": 518,
 "formatte": 519,
 "forward": 520,
 "fragmen": 521,
 "frame-inde": 528,
 "frame-spa": 532,
 "frame-val": 533,
 "from-c": 538,
 "from-cur": 539,
 "from-p": 540,
 "full-height": 541,
 "full-height-p": 542,
 "full-pathn": 543,
 "full-width": 544,
 "full-width-p": 545,
 "gateway": 548,
 "get-blue": 558,
 "get-file-offse": 579,
 "get-green": 582,
 "get-key-val": 587,
 "get-red": 599,
 "get-selected": 602,
 "get-text-height": 608,
 "get-text-height-p": 609,
 "get-text-width": 610,
 "get-text-width-p": 611,
 "go-pend": 625,
 "graphic-e": 627,
 "grid-factor-h": 628,
 "grid-factor-v": 629,
 "grid-unit-height": 631,
 "grid-unit-height-p": 632,
 "grid-unit-width": 633,
 "grid-unit-width-p": 634,
 "height": 1169,
 "height-p": 1170,
 "hori": 650,
 "image-size-c": 668,
 "image-size-p": 669,
 "info": 683,
 "inherit-bgc": 684,
 "inherit-fgc": 685,
 "init": 687,
 "input-o": 695,
 "insert-b": 699,
 "insert-t": 703,
 "is-attr": 714,
 "is-clas": 712,
 "is-partitione": 717,
 "keep-frame-z": 727,
 "keyfunc": 736,
 "key-func": 733,
 "label-bgc": 742,
 "label-dc": 743,
 "label-fgc": 744,
 "label-pfc": 746,
 "language": 750,
 "last-even": 757,
 "last-proce": 762,
 "last-tab-i": 764,
 "left-align": 769,
 "line-count": 776,
 "listi": 784,
 "load-mouse-p": 794,
 "margin-height": 821,
 "margin-height-p": 822,
 "margin-width": 823,
 "margin-width-p": 824,
 "max-height-c": 832,
 "max-height-p": 833,
 "max": 840,
 "max-val": 836,
 "max-width": 837,
 "max-width-p": 838,
 "menu-k": 849,
 "menu-m": 850,
 "min-column-width-c": 859,
 "min-column-width-p": 860,
 "min-height": 861,
 "min-height-p": 862,
 "min": 867,
 "min-val": 864,
 "min-width": 865,
 "min-width-p": 866,
 "mod": 869,
 "mouse-p": 872,
 "move-after": 874,
 "move-befor": 875,
 "move-col": 876,
 "move-to-b": 877,
 "move-to-t": 879,
 "next-tab-i": 901,
 "no-attr": 908,
 "no-f": 921,
 "no-inherit-bgc": 926,
 "no-inherit-fgc": 927,
 "no-label": 929,
 "no-mes": 934,
 "no-prefe": 936,
 "no-und": 942,
 "no-val": 944,
 "num-ali": 955,
 "num-but": 957,
 "num-col": 958,
 "numeric-f": 980,
 "num-locked-col": 968,
 "num-selected": 975,
 "on-frame": 988,
 "os-drive": 1001,
 "page-bot": 1011,
 "page-num": 1012,
 "page-wid": 1015,
 "pbe-hash-alg": 1025,
 "persist": 1028,
 "pfc": 1030,
 "pixels-per-col": 1032,
 "popup-m": 1034,
 "popup-o": 1035,
 "preproc": 1042,
 "presel": 1043,
 "prev-tab-i": 1047,
 "private-d": 1056,
 "proce": 1062,
 "proc-ha": 1058,
 "proc-st": 1059,
 "progress-s": 1069,
 "prompt-f": 1071,
 "provers": 1076,
 "put-key-val": 1089,
 "rcode-info": 1110,
 "resiza": 1144,
 "return-ins": 1154,
 "return-to-start-di": 1155,
 "return-val": 1156,
 "return-align": 1163,
 "sax-comple": 1182,
 "screen-val": 1204,
 "scrollbar-h": 1213,
 "scrollbar-v": 1214,
 "scrolled-row-pos": 1215,
 "scroll-to-i": 1210,
 "send-sql": 1241,
 "set-blue": 1261,
 "set-green": 1272,
 "set-red": 1279,
 "setuser": 1287,
 "share": 1289,
 "show-stat": 1292,
 "side-label-h": 1293,
 "side-lab": 1294,
 "size-c": 1302,
 "size-p": 1303,
 "stoppe": 1336,
 "stored-proc": 1337,
 "sub-ave": 1347,
 "sum-max": 1349,
 "sub-min": 1351,
 "subst": 1354,
 "substr": 1355,
 "suppress-w": 1361,
 "system-alert": 1366,
 "temp-dir": 1378,
 "term": 1381,
 "title-bgc": 1399,
 "title-dc": 1400,
 "title-fgc": 1401,
 "title-fo": 1402,
 "trunc": 1423,
 "unbuff": 1427,
 "underl": 1428,
 "unform": 1430,
 "vert": 1476,
 "virtual-height": 1480,
 "virtual-height-p": 1481,
 "virtual-width": 1482,
 "virtual-width-p": 1483,
 "widget-e": 1495,
 "widget-l": 1497,
 "width": 1499,
 "width-p": 1500,
 "window-maxim": 1502,
 "window-minim": 1503,
 "window-sta": 1506,
 "&globa": 3,
 "&global": 3,
 "&global-": 3,
 "&global-d": 3,
 "&global-de": 3,
 "&global-def": 3,
 "&global-defi": 3,
 "&global-defin": 3,
 "&scope": 6,
 "&scoped": 6,
 "&scoped-": 6,
 "&scoped-d": 6,
 "&scoped-de": 6,
 "&scoped-def": 6,
 "&scoped-defi": 6,
 "&scoped-defin": 6,
 "&undefi": 8,
 "&undefin": 8,
 "abso": 28,
 "absol": 28,
 "absolu": 28,
 "absolut": 28,
 "accumu": 31,
 "accumul": 31,
 "accumula": 31,
 "accumulat": 31,
 "ambigu": 59,
 "ambiguo": 59,
 "ambiguou": 59,
 "appl-alert-": 66,
 "appl-alert-b": 66,
 "appl-alert-bo": 66,
 "appl-alert-box": 66,
 "appl-alert-boxe": 66,
 "asce": 75,
 "ascen": 75,
 "ascend": 75,
 "ascendi": 75,
 "ascendin": 75,
 "attr-": 84,
 "attr-s": 84,
 "attr-sp": 84,
 "attr-spa": 84,
 "attr-spac": 84,
 "auto-compl": 91,
 "auto-comple": 91,
 "auto-complet": 91,
 "auto-completi": 91,
 "auto-completio": 91,
 "auto-inde": 95,
 "auto-inden": 95,
 "auto-retu": 97,
 "auto-retur": 97,
 "auto-za": 99,
 "availa": 101,
 "availab": 101,
 "availabl": 101,
 "aver": 103,
 "avera": 103,
 "averag": 103,
 "backg": 105,
 "backgr": 105,
 "backgro": 105,
 "backgrou": 105,
 "backgroun": 105,
 "batch-": 111,
 "batch-m": 111,
 "batch-mo": 111,
 "batch-mod": 111,
 "before-hi": 113,
 "before-hid": 113,
 "bgco": 118,
 "bgcol": 118,
 "bgcolo": 118,
 "block-leve": 125,
 "border-bo": 126,
 "border-bot": 126,
 "border-bott": 126,
 "border-botto": 126,
 "border-bottom": 126,
 "border-bottom-": 126,
 "border-bottom-c": 126,
 "border-bottom-ch": 126,
 "border-bottom-cha": 126,
 "border-bottom-char": 126,
 "border-bottom-pi": 127,
 "border-bottom-pix": 127,
 "border-bottom-pixe": 127,
 "border-bottom-pixel": 127,
 "border-le": 128,
 "border-lef": 128,
 "border-left": 128,
 "border-left-": 128,
 "border-left-c": 128,
 "border-left-ch": 128,
 "border-left-cha": 128,
 "border-left-char": 128,
 "border-left-pi": 129,
 "border-left-pix": 129,
 "border-left-pixe": 129,
 "border-left-pixel": 129,
 "border-ri": 130,
 "border-rig": 130,
 "border-righ": 130,
 "border-right": 130,
 "border-right-": 130,
 "border-right-c": 130,
 "border-right-ch": 130,
 "border-right-cha": 130,
 "border-right-char": 130,
 "border-right-pi": 131,
 "border-right-pix": 131,
 "border-right-pixe": 131,
 "border-right-pixel": 131,
 "border-to": 132,
 "border-top": 132,
 "border-top-": 132,
 "border-top-c": 132,
 "border-top-ch": 132,
 "border-top-cha": 132,
 "border-top-char": 132,
 "border-top-pi": 133,
 "border-top-pix": 133,
 "border-top-pixe": 133,
 "border-top-pixel": 133,
 "box-selecta": 135,
 "box-selectab": 135,
 "box-selectabl": 135,
 "case-sens": 174,
 "case-sensi": 174,
 "case-sensit": 174,
 "case-sensiti": 174,
 "case-sensitiv": 174,
 "centere": 178,
 "clear-selecti": 192,
 "clear-selectio": 192,
 "colon-aligne": 209,
 "colu": 212,
 "colum": 212,
 "column-labe": 217,
 "convert-to-offse": 249,
 "current-envi": 275,
 "current-envir": 275,
 "current-enviro": 275,
 "current-environ": 275,
 "current-environm": 275,
 "current-environme": 275,
 "current-environmen": 275,
 "current-langu": 277,
 "current-langua": 277,
 "current-languag": 277,
 "curso": 286,
 "data-entry-retu": 291,
 "data-entry-retur": 291,
 "data-rela": 292,
 "data-relat": 292,
 "data-relati": 292,
 "data-relatio": 292,
 "data-ty": 297,
 "data-typ": 297,
 "date-fo": 303,
 "date-for": 303,
 "date-form": 303,
 "date-forma": 303,
 "dbrestr": 313,
 "dbrestri": 313,
 "dbrestric": 313,
 "dbrestrict": 313,
 "dbrestricti": 313,
 "dbrestrictio": 313,
 "dbrestriction": 313,
 "dbversi": 316,
 "dbversio": 316,
 "default-ext": 338,
 "default-exte": 338,
 "default-exten": 338,
 "default-extens": 338,
 "default-extensi": 338,
 "default-extensio": 338,
 "default-noxla": 339,
 "default-noxlat": 339,
 "defi": 342,
 "defin": 342,
 "dele": 346,
 "delet": 346,
 "desce": 355,
 "descen": 355,
 "descend": 355,
 "descendi": 355,
 "descendin": 355,
 "dicti": 362,
 "dictio": 362,
 "diction": 362,
 "dictiona": 362,
 "dictionar": 362,
 "disconn": 369,
 "disconne": 369,
 "disconnec": 369,
 "displ": 370,
 "displa": 370,
 "display-ty": 372,
 "display-typ": 372,
 "edge-": 396,
 "edge-c": 396,
 "edge-ch": 396,
 "edge-cha": 396,
 "edge-char": 396,
 "edge-pi": 397,
 "edge-pix": 397,
 "edge-pixe": 397,
 "edge-pixel": 397,
 "error-colu": 432,
 "error-colum": 432,
 "error-statu": 435,
 "event-ty": 442,
 "event-typ": 442,
 "exclusive-": 446,
 "exclusive-l": 446,
 "exclusive-lo": 446,
 "exclusive-loc": 446,
 "fgco": 462,
 "fgcol": 462,
 "fgcolo": 462,
 "file-infor": 467,
 "file-inform": 467,
 "file-informa": 467,
 "file-informat": 467,
 "file-informati": 467,
 "file-informatio": 467,
 "file-offs": 471,
 "file-offse": 471,
 "first-proce": 501,
 "first-proced": 501,
 "first-procedu": 501,
 "first-procedur": 501,
 "first-tab-it": 503,
 "first-tab-ite": 503,
 "foreg": 515,
 "foregr": 515,
 "foregro": 515,
 "foregrou": 515,
 "foregroun": 515,
 "forma": 518,
 "frame-spac": 532,
 "frame-spaci": 532,
 "frame-spacin": 532,
 "frame-valu": 533,
 "from-ch": 538,
 "from-cha": 538,
 "from-char": 538,
 "from-curr": 539,
 "from-curre": 539,
 "from-curren": 539,
 "from-pi": 540,
 "from-pix": 540,
 "from-pixe": 540,
 "from-pixel": 540,
 "full-height-": 541,
 "full-height-c": 541,
 "full-height-ch": 541,
 "full-height-cha": 541,
 "full-height-char": 541,
 "full-height-pi": 542,
 "full-height-pix": 542,
 "full-height-pixe": 542,
 "full-height-pixel": 542,
 "full-pathna": 543,
 "full-pathnam": 543,
 "full-width-": 544,
 "full-width-c": 544,
 "full-width-ch": 544,
 "full-width-cha": 544,
 "full-width-char": 544,
 "full-width-pi": 545,
 "full-width-pix": 545,
 "full-width-pixe": 545,
 "full-width-pixel": 545,
 "get-blue-": 558,
 "get-blue-v": 558,
 "get-blue-va": 558,
 "get-blue-val": 558,
 "get-blue-valu": 558,
 "get-green-": 582,
 "get-green-v": 582,
 "get-green-va": 582,
 "get-green-val": 582,
 "get-green-valu": 582,
 "get-key-valu": 587,
 "get-red-": 599,
 "get-red-v": 599,
 "get-red-va": 599,
 "get-red-val": 599,
 "get-red-valu": 599,
 "get-selected-": 602,
 "get-selected-w": 602,
 "get-selected-wi": 602,
 "get-selected-wid": 602,
 "get-selected-widg": 602,
 "get-selected-widge": 602,
 "get-text-height-": 608,
 "get-text-height-c": 608,
 "get-text-height-ch": 608,
 "get-text-height-cha": 608,
 "get-text-height-char": 608,
 "get-text-height-pi": 609,
 "get-text-height-pix": 609,
 "get-text-height-pixe": 609,
 "get-text-height-pixel": 609,
 "get-text-width-": 610,
 "get-text-width-c": 610,
 "get-text-width-ch": 610,
 "get-text-width-cha": 610,
 "get-text-width-char": 610,
 "get-text-width-pi": 611,
 "get-text-width-pix": 611,
 "get-text-width-pixe": 611,
 "get-text-width-pixel": 611,
 "go-pendi": 625,
 "go-pendin": 625,
 "graphic-ed": 627,
 "graphic-edg": 627,
 "grid-factor-ho": 628,
 "grid-factor-hor": 628,
 "grid-factor-hori": 628,
 "grid-factor-horiz": 628,
 "grid-factor-horizo": 628,
 "grid-factor-horizon": 628,
 "grid-factor-horizont": 628,
 "grid-factor-horizonta": 628,
 "grid-factor-ve": 629,
 "grid-factor-ver": 629,
 "grid-factor-vert": 629,
 "grid-factor-verti": 629,
 "grid-factor-vertic": 629,
 "grid-factor-vertica": 629,
 "grid-unit-height-": 631,
 "grid-unit-height-c": 631,
 "grid-unit-height-ch": 631,
 "grid-unit-height-cha": 631,
 "grid-unit-height-char": 631,
 "grid-unit-height-pi": 632,
 "grid-unit-height-pix": 632,
 "grid-unit-height-pixe": 632,
 "grid-unit-height-pixel": 632,
 "grid-unit-width-": 633,
 "grid-unit-width-c": 633,
 "grid-unit-width-ch": 633,
 "grid-unit-width-cha": 633,
 "grid-unit-width-char": 633,
 "grid-unit-width-pi": 634,
 "grid-unit-width-pix": 634,
 "grid-unit-width-pixe": 634,
 "grid-unit-width-pixel": 634,
 "horiz": 650,
 "horizo": 650,
 "horizon": 650,
 "horizont": 650,
 "horizonta": 650,
 "image-size-ch": 668,
 "image-size-cha": 668,
 "image-size-char": 668,
 "image-size-pi": 669,
 "image-size-pix": 669,
 "image-size-pixe": 669,
 "image-size-pixel": 669,
 "infor": 683,
 "inform": 683,
 "informa": 683,
 "informat": 683,
 "informati": 683,
 "informatio": 683,
 "inherit-bgco": 684,
 "inherit-bgcol": 684,
 "inherit-bgcolo": 684,
 "inherit-fgco": 685,
 "inherit-fgcol": 685,
 "inherit-fgcolo": 685,
 "initi": 687,
 "initia": 687,
 "input-ou": 695,
 "input-out": 695,
 "input-outp": 695,
 "input-outpu": 695,
 "insert-ba": 699,
 "insert-bac": 699,
 "insert-back": 699,
 "insert-backt": 699,
 "insert-backta": 699,
 "insert-ta": 703,
 "keep-frame-z-": 727,
 "keep-frame-z-o": 727,
 "keep-frame-z-or": 727,
 "keep-frame-z-ord": 727,
 "keep-frame-z-orde": 727,
 "keyfunct": 736,
 "keyfuncti": 736,
 "keyfunctio": 736,
 "key-funct": 733,
 "key-functi": 733,
 "key-functio": 733,
 "label-bgco": 742,
 "label-bgcol": 742,
 "label-bgcolo": 742,
 "label-dco": 743,
 "label-dcol": 743,
 "label-dcolo": 743,
 "label-fgco": 744,
 "label-fgcol": 744,
 "label-fgcolo": 744,
 "label-pfco": 746,
 "label-pfcol": 746,
 "label-pfcolo": 746,
 "last-proced": 762,
 "last-procedu": 762,
 "last-procedur": 762,
 "last-tab-it": 764,
 "last-tab-ite": 764,
 "left-aligne": 769,
 "line-counte": 776,
 "listin": 784,
 "load-mouse-po": 794,
 "load-mouse-poi": 794,
 "load-mouse-poin": 794,
 "load-mouse-point": 794,
 "load-mouse-pointe": 794,
 "margin-height-": 821,
 "margin-height-c": 821,
 "margin-height-ch": 821,
 "margin-height-cha": 821,
 "margin-height-char": 821,
 "margin-height-pi": 822,
 "margin-height-pix": 822,
 "margin-height-pixe": 822,
 "margin-height-pixel": 822,
 "margin-width-": 823,
 "margin-width-c": 823,
 "margin-width-ch": 823,
 "margin-width-cha": 823,
 "margin-width-char": 823,
 "margin-width-pi": 824,
 "margin-width-pix": 824,
 "margin-width-pixe": 824,
 "margin-width-pixel": 824,
 "max-height-ch": 832,
 "max-height-cha": 832,
 "max-height-char": 832,
 "max-height-pi": 833,
 "max-height-pix": 833,
 "max-height-pixe": 833,
 "max-height-pixel": 833,
 "maxi": 840,
 "maxim": 840,
 "maximu": 840,
 "max-valu": 836,
 "max-width-": 837,
 "max-width-c": 837,
 "max-width-ch": 837,
 "max-width-cha": 837,
 "max-width-char": 837,
 "max-width-pi": 838,
 "max-width-pix": 838,
 "max-width-pixe": 838,
 "max-width-pixel": 838,
 "menu-ke": 849,
 "menu-mo": 850,
 "menu-mou": 850,
 "menu-mous": 850,
 "min-column-width-ch": 859,
 "min-column-width-cha": 859,
 "min-column-width-char": 859,
 "min-column-width-pi": 860,
 "min-column-width-pix": 860,
 "min-column-width-pixe": 860,
 "min-column-width-pixel": 860,
 "min-height-": 861,
 "min-height-c": 861,
 "min-height-ch": 861,
 "min-height-cha": 861,
 "min-height-char": 861,
 "min-height-pi": 862,
 "min-height-pix": 862,
 "min-height-pixe": 862,
 "min-height-pixel": 862,
 "mini": 867,
 "minim": 867,
 "minimu": 867,
 "min-valu": 864,
 "min-width-": 865,
 "min-width-c": 865,
 "min-width-ch": 865,
 "min-width-cha": 865,
 "min-width-char": 865,
 "min-width-pi": 866,
 "min-width-pix": 866,
 "min-width-pixe": 866,
 "min-width-pixel": 866,
 "modu": 869,
 "modul": 869,
 "mouse-po": 872,
 "mouse-poi": 872,
 "mouse-poin": 872,
 "mouse-point": 872,
 "mouse-pointe": 872,
 "move-after-": 874,
 "move-after-t": 874,
 "move-after-ta": 874,
 "move-after-tab": 874,
 "move-after-tab-": 874,
 "move-after-tab-i": 874,
 "move-after-tab-it": 874,
 "move-after-tab-ite": 874,
 "move-before": 875,
 "move-before-": 875,
 "move-before-t": 875,
 "move-before-ta": 875,
 "move-before-tab": 875,
 "move-before-tab-": 875,
 "move-before-tab-i": 875,
 "move-before-tab-it": 875,
 "move-before-tab-ite": 875,
 "move-colu": 876,
 "move-colum": 876,
 "move-to-bo": 877,
 "move-to-bot": 877,
 "move-to-bott": 877,
 "move-to-botto": 877,
 "move-to-to": 879,
 "next-tab-it": 901,
 "next-tab-ite": 901,
 "no-attr-": 908,
 "no-attr-s": 908,
 "no-attr-sp": 908,
 "no-attr-spa": 908,
 "no-attr-spac": 908,
 "no-fi": 921,
 "no-fil": 921,
 "no-inherit-bgco": 926,
 "no-inherit-bgcol": 926,
 "no-inherit-bgcolo": 926,
 "no-inherit-fgco": 927,
 "no-inherit-fgcol": 927,
 "no-inherit-fgcolo": 927,
 "no-mess": 934,
 "no-messa": 934,
 "no-messag": 934,
 "no-prefet": 936,
 "no-prefetc": 936,
 "no-unde": 942,
 "no-under": 942,
 "no-underl": 942,
 "no-underli": 942,
 "no-underlin": 942,
 "no-vali": 944,
 "no-valid": 944,
 "no-valida": 944,
 "no-validat": 944,
 "num-alia": 955,
 "num-alias": 955,
 "num-aliase": 955,
 "num-butt": 957,
 "num-butto": 957,
 "num-button": 957,
 "num-colu": 958,
 "num-colum": 958,
 "num-column": 958,
 "numeric-fo": 980,
 "numeric-for": 980,
 "numeric-form": 980,
 "numeric-forma": 980,
 "num-locked-colu": 968,
 "num-locked-colum": 968,
 "num-locked-column": 968,
 "num-selected-": 975,
 "num-selected-w": 975,
 "num-selected-wi": 975,
 "num-selected-wid": 975,
 "num-selected-widg": 975,
 "num-selected-widge": 975,
 "num-selected-widget": 975,
 "on-frame-": 988,
 "on-frame-b": 988,
 "on-frame-bo": 988,
 "on-frame-bor": 988,
 "on-frame-bord": 988,
 "on-frame-borde": 988,
 "page-bott": 1011,
 "page-botto": 1011,
 "page-numb": 1012,
 "page-numbe": 1012,
 "page-widt": 1015,
 "pbe-hash-algo": 1025,
 "pbe-hash-algor": 1025,
 "pbe-hash-algori": 1025,
 "pbe-hash-algorit": 1025,
 "pbe-hash-algorith": 1025,
 "persiste": 1028,
 "persisten": 1028,
 "pfco": 1030,
 "pfcol": 1030,
 "pfcolo": 1030,
 "pixels-per-colu": 1032,
 "pixels-per-colum": 1032,
 "popup-me": 1034,
 "popup-men": 1034,
 "popup-on": 1035,
 "popup-onl": 1035,
 "preproce": 1042,
 "preproces": 1042,
 "presele": 1043,
 "preselec": 1043,
 "prev-tab-it": 1047,
 "prev-tab-ite": 1047,
 "private-da": 1056,
 "private-dat": 1056,
 "proced": 1062,
 "procedu": 1062,
 "procedur": 1062,
 "proc-han": 1058,
 "proc-hand": 1058,
 "proc-handl": 1058,
 "proc-sta": 1059,
 "proc-stat": 1059,
 "proc-statu": 1059,
 "progress-so": 1069,
 "progress-sou": 1069,
 "progress-sour": 1069,
 "progress-sourc": 1069,
 "prompt-fo": 1071,
 "proversi": 1076,
 "proversio": 1076,
 "put-key-valu": 1089,
 "rcode-infor": 1110,
 "rcode-inform": 1110,
 "rcode-informa": 1110,
 "rcode-informat": 1110,
 "rcode-informati": 1110,
 "rcode-informatio": 1110,
 "resizab": 1144,
 "resizabl": 1144,
 "return-inse": 1154,
 "return-inser": 1154,
 "return-insert": 1154,
 "return-inserte": 1154,
 "return-valu": 1156,
 "sax-complet": 1182,
 "screen-valu": 1204,
 "scrollbar-ho": 1213,
 "scrollbar-hor": 1213,
 "scrollbar-hori": 1213,
 "scrollbar-horiz": 1213,
 "scrollbar-horizo": 1213,
 "scrollbar-horizon": 1213,
 "scrollbar-horizont": 1213,
 "scrollbar-horizonta": 1213,
 "scrollbar-ve": 1214,
 "scrollbar-ver": 1214,
 "scrollbar-vert": 1214,
 "scrollbar-verti": 1214,
 "scrollbar-vertic": 1214,
 "scrollbar-vertica": 1214,
 "scrolled-row-posi": 1215,
 "scrolled-row-posit": 1215,
 "scrolled-row-positi": 1215,
 "scrolled-row-positio": 1215,
 "scroll-to-it": 1210,
 "scroll-to-ite": 1210,
 "send-sql-": 1241,
 "send-sql-s": 1241,
 "send-sql-st": 1241,
 "send-sql-sta": 1241,
 "send-sql-stat": 1241,
 "send-sql-state": 1241,
 "send-sql-statem": 1241,
 "send-sql-stateme": 1241,
 "send-sql-statemen": 1241,
 "set-blue-": 1261,
 "set-blue-v": 1261,
 "set-blue-va": 1261,
 "set-blue-val": 1261,
 "set-blue-valu": 1261,
 "set-green-": 1272,
 "set-green-v": 1272,
 "set-green-va": 1272,
 "set-green-val": 1272,
 "set-green-valu": 1272,
 "set-red-": 1279,
 "set-red-v": 1279,
 "set-red-va": 1279,
 "set-red-val": 1279,
 "set-red-valu": 1279,
 "setuseri": 1287,
 "share-": 1289,
 "share-l": 1289,
 "share-lo": 1289,
 "share-loc": 1289,
 "side-label-ha": 1293,
 "side-label-han": 1293,
 "side-label-hand": 1293,
 "side-label-handl": 1293,
 "side-labe": 1294,
 "side-label": 1294,
 "size-ch": 1302,
 "size-cha": 1302,
 "size-char": 1302,
 "size-pi": 1303,
 "size-pix": 1303,
 "size-pixe": 1303,
 "size-pixel": 1303,
 "stored-proce": 1337,
 "stored-proced": 1337,
 "stored-procedu": 1337,
 "stored-procedur": 1337,
 "sub-aver": 1347,
 "sub-avera": 1347,
 "sub-averag": 1347,
 "sub-mini": 1351,
 "sub-minim": 1351,
 "sub-minimu": 1351,
 "substi": 1354,
 "substit": 1354,
 "substitu": 1354,
 "substitut": 1354,
 "substri": 1355,
 "substrin": 1355,
 "suppress-wa": 1361,
 "suppress-war": 1361,
 "suppress-warn": 1361,
 "suppress-warni": 1361,
 "suppress-warnin": 1361,
 "suppress-warning": 1361,
 "system-alert-": 1366,
 "system-alert-b": 1366,
 "system-alert-bo": 1366,
 "system-alert-box": 1366,
 "system-alert-boxe": 1366,
 "temp-dire": 1378,
 "temp-direc": 1378,
 "temp-direct": 1378,
 "temp-directo": 1378,
 "temp-director": 1378,
 "termi": 1381,
 "termin": 1381,
 "termina": 1381,
 "title-bgco": 1399,
 "title-bgcol": 1399,
 "title-bgcolo": 1399,
 "title-dco": 1400,
 "title-dcol": 1400,
 "title-dcolo": 1400,
 "title-fgco": 1401,
 "title-fgcol": 1401,
 "title-fgcolo": 1401,
 "title-fon": 1402,
 "trunca": 1423,
 "truncat": 1423,
 "unbuffe": 1427,
 "unbuffer": 1427,
 "unbuffere": 1427,
 "underli": 1428,
 "underlin": 1428,
 "unforma": 1430,
 "unformat": 1430,
 "unformatt": 1430,
 "unformatte": 1430,
 "verti": 1476,
 "vertic": 1476,
 "vertica": 1476,
 "virtual-height-": 1480,
 "virtual-height-c": 1480,
 "virtual-height-ch": 1480,
 "virtual-height-cha": 1480,
 "virtual-height-char": 1480,
 "virtual-height-pi": 1481,
 "virtual-height-pix": 1481,
 "virtual-height-pixe": 1481,
 "virtual-height-pixel": 1481,
 "virtual-width-": 1482,
 "virtual-width-c": 1482,
 "virtual-width-ch": 1482,
 "virtual-width-cha": 1482,
 "virtual-width-char": 1482,
 "virtual-width-pi": 1483,
 "virtual-width-pix": 1483,
 "virtual-width-pixe": 1483,
 "virtual-width-pixel": 1483,
 "widget-en": 1495,
 "widget-ent": 1495,
 "widget-ente": 1495,
 "widget-le": 1497,
 "widget-lea": 1497,
 "widget-leav": 1497,
 "width-": 1499,
 "width-c": 1499,
 "width-ch": 1499,
 "width-cha": 1499,
 "width-char": 1499,
 "width-pi": 1500,
 "width-pix": 1500,
 "width-pixe": 1500,
 "width-pixel": 1500,
 "window-maximi": 1502,
 "window-maximiz": 1502,
 "window-maximize": 1502,
 "window-minimi": 1503,
 "window-minimiz": 1503,
 "window-minimize": 1503,
 "window-stat": 1506,
 "+": 12,
 "-": 13,
 ".": 14,
 "&else": 0,
 "&elseif": 1,
 "&endif": 2,
 "&global-define": 3,
 "&if": 4,
 "&message": 5,
 "&scoped-define": 6,
 "&then": 7,
 "&undefine": 8,
 "&webstream": 9,
 "*": 11,
 "/": 15,
 ":": 16,
 "?": 23,
 "@": 24,
 "[": 25,
 "]": 26,
 "^": 27,
 "{&batch-mode}": 1550,
 "{&file-name}": 1551,
 "{&line-number}": 1552,
 "{&opsys}": 1553,
 "{&process-architecture}": 1554,
 "{&sequence}": 1555,
 "{&window-system}": 1556,
 "'": 10,
 "<": 17,
 "<=": 18,
 "<>": 19,
 "=": 20,
 ">": 21,
 ">=": 22,
 "absolute": 28,
 "abstract": 29,
 "accelerator": 30,
 "accumulate": 31,
 "active-form": 32,
 "active-window": 33,
 "add": 34,
 "add-buffer": 35,
 "add-calc-column": 36,
 "add-columns-from": 37,
 "add-events-procedure": 38,
 "add-fields-from": 39,
 "add-first": 40,
 "add-index-field": 41,
 "add-last": 42,
 "add-like-column": 43,
 "add-like-field": 44,
 "add-like-index": 45,
 "add-new-field": 46,
 "add-new-index": 47,
 "add-schema-location": 48,
 "add-super-procedure": 49,
 "adm-data": 50,
 "advise": 51,
 "alert-box": 52,
 "alias": 53,
 "all": 54,
 "allow-column-searching": 55,
 "allow-replication": 56,
 "alter": 57,
 "always-on-top": 58,
 "ambiguous": 59,
 "analyze": 60,
 "and": 61,
 "ansi-only": 62,
 "any": 63,
 "anywhere": 64,
 "append": 65,
 "appl-alert-boxes": 66,
 "appl-context-id": 67,
 "application": 68,
 "apply": 69,
 "appserver-info": 70,
 "appserver-password": 71,
 "appserver-userid": 72,
 "array-message": 73,
 "as": 74,
 "ascending": 75,
 "ask-overwrite": 76,
 "assembly": 77,
 "assign": 78,
 "asynchronous": 81,
 "async-request-count": 79,
 "async-request-handle": 80,
 "at": 82,
 "attached-pairlist": 83,
 "attr-space": 84,
 "audit-control": 85,
 "audit-enabled": 86,
 "audit-event-context": 87,
 "audit-policy": 88,
 "authentication-failed": 89,
 "authorization": 90,
 "auto-completion": 91,
 "auto-endkey": 93,
 "auto-end-key": 92,
 "auto-go": 94,
 "auto-indent": 95,
 "automatic": 100,
 "auto-resize": 96,
 "auto-return": 97,
 "auto-synchronize": 98,
 "auto-zap": 99,
 "available": 101,
 "available-formats": 102,
 "average": 103,
 "avg": 104,
 "background": 105,
 "backwards": 106,
 "base64-decode": 109,
 "base64-encode": 110,
 "base-ade": 107,
 "base-key": 108,
 "batch-mode": 111,
 "batch-size": 112,
 "before-hide": 113,
 "begin-event-group": 114,
 "begins": 115,
 "bell": 116,
 "between": 117,
 "bgcolor": 118,
 "big-endian": 119,
 "binary": 120,
 "bind": 121,
 "bind-where": 122,
 "blank": 123,
 "block-iteration-display": 124,
 "block-level": 125,
 "border-bottom-chars": 126,
 "border-bottom-pixels": 127,
 "border-left-chars": 128,
 "border-left-pixels": 129,
 "border-right-chars": 130,
 "border-right-pixels": 131,
 "border-top-chars": 132,
 "border-top-pixels": 133,
 "box": 134,
 "box-selectable": 135,
 "break": 136,
 "buffer-chars": 139,
 "buffer-compare": 140,
 "buffer-copy": 141,
 "buffer-create": 142,
 "buffer-delete": 143,
 "buffer-field": 144,
 "buffer-handle": 145,
 "buffer-lines": 146,
 "buffer-name": 147,
 "buffer-partition-id": 148,
 "buffer-release": 149,
 "buffer-value": 150,
 "by": 152,
 "by-pointer": 153,
 "by-variant-pointer": 154,
 "cache": 155,
 "cache-size": 156,
 "call": 157,
 "call-name": 158,
 "call-type": 159,
 "cancel-break": 169,
 "cancel-button": 170,
 "can-create": 160,
 "can-delete": 161,
 "can-do": 162,
 "can-do-domain-support": 163,
 "can-find": 164,
 "can-query": 165,
 "can-read": 166,
 "can-set": 167,
 "can-write": 168,
 "caps": 171,
 "careful-paint": 172,
 "case": 173,
 "case-sensitive": 174,
 "cast": 175,
 "catch": 176,
 "cdecl": 177,
 "centered": 178,
 "chained": 179,
 "character_length": 181,
 "charset": 182,
 "check": 183,
 "checked": 184,
 "choose": 185,
 "chr": 186,
 "class": 187,
 "class-type": 188,
 "clear": 189,
 "clear-appl-context": 190,
 "clear-log": 191,
 "clear-selection": 192,
 "clear-sort-arrows": 193,
 "client-connection-id": 194,
 "client-principal": 195,
 "client-tty": 196,
 "client-type": 197,
 "client-workstation": 198,
 "clipboard": 199,
 "close": 200,
 "close-log": 201,
 "code": 202,
 "codebase-locator": 203,
 "codepage": 204,
 "codepage-convert": 205,
 "collate": 207,
 "col-of": 206,
 "colon": 208,
 "colon-aligned": 209,
 "color": 210,
 "color-table": 211,
 "column": 212,
 "column-bgcolor": 213,
 "column-dcolor": 214,
 "column-fgcolor": 215,
 "column-font": 216,
 "column-label": 217,
 "column-movable": 218,
 "column-of": 219,
 "column-pfcolor": 220,
 "column-read-only": 221,
 "column-resizable": 222,
 "columns": 224,
 "column-scrolling": 223,
 "combo-box": 227,
 "command": 228,
 "compares": 229,
 "compile": 230,
 "compiler": 231,
 "complete": 232,
 "com-self": 226,
 "config-name": 233,
 "connect": 234,
 "connected": 235,
 "constructor": 236,
 "contains": 237,
 "contents": 238,
 "context": 239,
 "context-help": 240,
 "context-help-file": 241,
 "context-help-id": 242,
 "context-popup": 243,
 "control": 244,
 "control-box": 245,
 "control-frame": 246,
 "convert": 247,
 "convert-3d-colors": 248,
 "convert-to-offset": 249,
 "copy-dataset": 250,
 "copy-lob": 251,
 "copy-sax-attributes": 252,
 "copy-temp-table": 253,
 "count": 254,
 "count-of": 255,
 "cpcase": 256,
 "cpcoll": 257,
 "cpinternal": 258,
 "cplog": 259,
 "cpprint": 260,
 "cprcodein": 261,
 "cprcodeout": 262,
 "cpstream": 263,
 "cpterm": 264,
 "crc-value": 265,
 "create": 266,
 "create-like": 267,
 "create-like-sequential": 268,
 "create-node-namespace": 269,
 "create-result-list-entry": 270,
 "create-test-file": 271,
 "current": 272,
 "current_date": 285,
 "current-changed": 273,
 "current-column": 274,
 "current-environment": 275,
 "current-iteration": 276,
 "current-language": 277,
 "current-query": 278,
 "current-request-info": 279,
 "current-response-info": 280,
 "current-result-row": 281,
 "current-row-modified": 282,
 "current-value": 283,
 "current-window": 284,
 "cursor": 286,
 "cursor-char": 287,
 "cursor-line": 288,
 "cursor-offset": 289,
 "database": 298,
 "data-bind": 290,
 "data-entry-return": 291,
 "data-relation": 292,
 "dataservers": 299,
 "dataset-handle": 301,
 "data-source-complete-map": 294,
 "data-source-modified": 295,
 "data-source-rowid": 296,
 "data-type": 297,
 "date-format": 303,
 "day": 306,
 "dbcodepage": 309,
 "dbcollation": 310,
 "dbname": 311,
 "dbparam": 312,
 "db-context": 307,
 "db-references": 308,
 "dbrestrictions": 313,
 "dbtaskid": 314,
 "dbtype": 315,
 "dbversion": 316,
 "dcolor": 317,
 "dde": 318,
 "dde-error": 319,
 "dde-id": 320,
 "dde-item": 321,
 "dde-name": 322,
 "dde-topic": 323,
 "deblank": 324,
 "debug": 325,
 "debug-alert": 326,
 "debugger": 328,
 "debug-list": 327,
 "decimals": 330,
 "declare": 331,
 "declare-namespace": 332,
 "decrypt": 333,
 "default": 334,
 "default-buffer-handle": 335,
 "default-button": 336,
 "default-commit": 337,
 "default-extension": 338,
 "default-noxlate": 339,
 "default-value": 340,
 "default-window": 341,
 "define": 342,
 "defined": 344,
 "define-user-event-manager": 343,
 "delegate": 345,
 "delete": 346,
 "delete procedure": 347,
 "delete-char": 348,
 "delete-current-row": 349,
 "delete-line": 350,
 "delete-result-list-entry": 351,
 "delete-selected-row": 352,
 "delete-selected-rows": 353,
 "delimiter": 354,
 "descending": 355,
 "deselect-focused-row": 356,
 "deselection": 359,
 "deselect-rows": 357,
 "deselect-selected-row": 358,
 "destructor": 360,
 "dialog-box": 361,
 "dictionary": 362,
 "dir": 363,
 "disable": 364,
 "disable-auto-zap": 365,
 "disabled": 368,
 "disable-dump-triggers": 366,
 "disable-load-triggers": 367,
 "disconnect": 369,
 "display": 370,
 "display-message": 371,
 "display-type": 372,
 "distinct": 373,
 "do": 374,
 "domain-description": 375,
 "domain-name": 376,
 "domain-type": 377,
 "dos": 378,
 "double": 379,
 "down": 380,
 "drag-enabled": 381,
 "drop": 382,
 "drop-down": 383,
 "drop-down-list": 384,
 "drop-file-notify": 385,
 "drop-target": 386,
 "ds-close-cursor": 387,
 "dslog-manager": 388,
 "dump": 389,
 "dynamic": 390,
 "dynamic-enum": 391,
 "dynamic-function": 392,
 "dynamic-invoke": 393,
 "each": 394,
 "echo": 395,
 "edge-chars": 396,
 "edge-pixels": 397,
 "edit-can-paste": 398,
 "edit-can-undo": 399,
 "edit-clear": 400,
 "edit-copy": 401,
 "edit-cut": 402,
 "editing": 405,
 "editor": 406,
 "edit-paste": 403,
 "edit-undo": 404,
 "else": 407,
 "empty": 408,
 "empty-temp-table": 409,
 "enable": 410,
 "enabled-fields": 411,
 "encode": 412,
 "encrypt": 413,
 "encrypt-audit-mac-key": 414,
 "encryption-salt": 415,
 "end": 416,
 "end-document": 417,
 "end-element": 418,
 "end-event-group": 419,
 "end-file-drop": 420,
 "endkey": 426,
 "end-key": 421,
 "end-move": 422,
 "end-resize": 423,
 "end-row-resize": 424,
 "end-user-prompt": 425,
 "entered": 427,
 "entity-expansion-limit": 428,
 "entry": 429,
 "eq": 20,
 "error": 431,
 "error-column": 432,
 "error-row": 433,
 "error-stack-trace": 434,
 "error-status": 435,
 "escape": 436,
 "etime": 437,
 "event-group-id": 439,
 "event-procedure": 440,
 "event-procedure-context": 441,
 "events": 443,
 "event-type": 442,
 "except": 444,
 "exclusive-id": 445,
 "exclusive-lock": 446,
 "exclusive-web-user": 447,
 "execute": 448,
 "exists": 449,
 "exp": 450,
 "expand": 451,
 "expandable": 452,
 "explicit": 453,
 "export": 454,
 "export-principal": 455,
 "extended": 456,
 "extent": 457,
 "external": 458,
 "false": 459,
 "fetch": 460,
 "fetch-selected-row": 461,
 "fgcolor": 462,
 "fields": 463,
 "file": 464,
 "file-create-date": 465,
 "file-create-time": 466,
 "file-information": 467,
 "file-mod-date": 468,
 "file-mod-time": 469,
 "filename": 474,
 "file-name": 470,
 "file-offset": 471,
 "file-size": 472,
 "file-type": 473,
 "fill": 475,
 "filled": 477,
 "fill-in": 476,
 "filters": 478,
 "final": 479,
 "finally": 480,
 "find": 481,
 "find-by-rowid": 482,
 "find-case-sensitive": 483,
 "find-current": 484,
 "finder": 493,
 "find-first": 485,
 "find-global": 486,
 "find-last": 487,
 "find-next-occurrence": 488,
 "find-prev-occurrence": 489,
 "find-select": 490,
 "find-unique": 491,
 "find-wrap-around": 492,
 "first": 494,
 "first-asynch-request": 495,
 "first-child": 496,
 "first-column": 497,
 "first-form": 498,
 "first-object": 499,
 "first-of": 500,
 "first-procedure": 501,
 "first-server": 502,
 "first-tab-item": 503,
 "fit-last-column": 504,
 "fixed-only": 505,
 "flat-button": 506,
 "float": 507,
 "focus": 508,
 "focused-row": 509,
 "focused-row-selected": 510,
 "font": 511,
 "font-table": 512,
 "for": 513,
 "force-file": 514,
 "foreground": 515,
 "form input": 516,
 "format": 518,
 "formatted": 519,
 "form-long-input": 517,
 "forwards": 520,
 "fragment": 521,
 "frame-col": 523,
 "frame-db": 524,
 "frame-down": 525,
 "frame-field": 526,
 "frame-file": 527,
 "frame-index": 528,
 "frame-line": 529,
 "frame-name": 530,
 "frame-row": 531,
 "frame-spacing": 532,
 "frame-value": 533,
 "frame-x": 534,
 "frame-y": 535,
 "frequency": 536,
 "from": 537,
 "from-chars": 538,
 "from-current": 539,
 "from-pixels": 540,
 "full-height-chars": 541,
 "full-height-pixels": 542,
 "full-pathname": 543,
 "full-width-chars": 544,
 "full-width-pixels": 545,
 "function": 546,
 "function-call-type": 547,
 "gateways": 548,
 "ge": 22,
 "generate-md5": 549,
 "generate-pbe-key": 550,
 "generate-pbe-salt": 551,
 "generate-random-key": 552,
 "generate-uuid": 553,
 "get": 554,
 "get-attr-call-type": 555,
 "get-attribute-node": 556,
 "get-binary-data": 557,
 "get-blue-value": 558,
 "get-browse-column": 559,
 "get-buffer-handle": 560,
 "getbyte": 622,
 "get-byte": 561,
 "get-callback-proc-context": 562,
 "get-callback-proc-name": 563,
 "get-cgi-list": 564,
 "get-cgi-long-value": 565,
 "get-cgi-value": 566,
 "get-class": 567,
 "get-codepages": 568,
 "get-collations": 569,
 "get-config-value": 570,
 "get-current": 571,
 "get-double": 572,
 "get-dropped-file": 573,
 "get-dynamic": 574,
 "get-error-column": 575,
 "get-error-row": 576,
 "get-file": 577,
 "get-file-name": 578,
 "get-file-offset": 579,
 "get-first": 580,
 "get-float": 581,
 "get-green-value": 582,
 "get-index-by-namespace-name": 583,
 "get-index-by-qname": 584,
 "get-int64": 585,
 "get-iteration": 586,
 "get-key-value": 587,
 "get-last": 588,
 "get-localname-by-index": 589,
 "get-long": 590,
 "get-message": 591,
 "get-next": 592,
 "get-number": 593,
 "get-pointer-value": 594,
 "get-prev": 595,
 "get-printers": 596,
 "get-property": 597,
 "get-qname-by-index": 598,
 "get-red-value": 599,
 "get-repositioned-row": 600,
 "get-rgb-value": 601,
 "get-selected-widget": 602,
 "get-short": 603,
 "get-signature": 604,
 "get-size": 605,
 "get-string": 606,
 "get-tab-item": 607,
 "get-text-height-chars": 608,
 "get-text-height-pixels": 609,
 "get-text-width-chars": 610,
 "get-text-width-pixels": 611,
 "get-type-by-index": 612,
 "get-type-by-namespace-name": 613,
 "get-type-by-qname": 614,
 "get-unsigned-long": 615,
 "get-unsigned-short": 616,
 "get-uri-by-index": 617,
 "get-value-by-index": 618,
 "get-value-by-namespace-name": 619,
 "get-value-by-qname": 620,
 "get-wait-state": 621,
 "global": 623,
 "go-on": 624,
 "go-pending": 625,
 "grant": 626,
 "graphic-edge": 627,
 "grid-factor-horizontal": 628,
 "grid-factor-vertical": 629,
 "grid-snap": 630,
 "grid-unit-height-chars": 631,
 "grid-unit-height-pixels": 632,
 "grid-unit-width-chars": 633,
 "grid-unit-width-pixels": 634,
 "grid-visible": 635,
 "group": 636,
 "gt": 21,
 "guid": 637,
 "handler": 639,
 "has-records": 640,
 "having": 641,
 "header": 642,
 "height-chars": 643,
 "height-pixels": 644,
 "help": 645,
 "hex-decode": 646,
 "hex-encode": 647,
 "hidden": 648,
 "hide": 649,
 "horizontal": 650,
 "host-byte-order": 651,
 "html-charset": 652,
 "html-end-of-line": 653,
 "html-end-of-page": 654,
 "html-frame-begin": 655,
 "html-frame-end": 656,
 "html-header-begin": 657,
 "html-header-end": 658,
 "html-title-begin": 659,
 "html-title-end": 660,
 "hwnd": 661,
 "icon": 662,
 "if": 663,
 "image-down": 665,
 "image-insensitive": 666,
 "image-size": 667,
 "image-size-chars": 668,
 "image-size-pixels": 669,
 "image-up": 670,
 "immediate-display": 671,
 "implements": 672,
 "import": 673,
 "import-principal": 674,
 "in": 675,
 "increment-exclusive-id": 677,
 "index": 678,
 "indexed-reposition": 681,
 "index-hint": 679,
 "index-information": 680,
 "indicator": 682,
 "information": 683,
 "in-handle": 676,
 "inherit-bgcolor": 684,
 "inherit-fgcolor": 685,
 "inherits": 686,
 "initial": 687,
 "initial-dir": 688,
 "initial-filter": 689,
 "initialize-document-type": 690,
 "initiate": 691,
 "inner-chars": 692,
 "inner-lines": 693,
 "input": 694,
 "input-output": 695,
 "input-value": 696,
 "insert": 697,
 "insert-attribute": 698,
 "insert-backtab": 699,
 "insert-file": 700,
 "insert-row": 701,
 "insert-string": 702,
 "insert-tab": 703,
 "interface": 706,
 "internal-entries": 707,
 "into": 708,
 "invoke": 709,
 "is": 710,
 "is-attr-space": 711,
 "is-class": 712,
 "is-json": 713,
 "is-lead-byte": 714,
 "is-open": 715,
 "is-parameter-set": 716,
 "is-partitioned": 717,
 "is-row-selected": 718,
 "is-selected": 719,
 "is-xml": 720,
 "item": 721,
 "items-per-row": 722,
 "join": 723,
 "join-by-sqldb": 724,
 "kblabel": 725,
 "keep-connection-open": 726,
 "keep-frame-z-order": 727,
 "keep-messages": 728,
 "keep-security-cache": 729,
 "keep-tab-order": 730,
 "key": 731,
 "keycode": 735,
 "key-code": 732,
 "keyfunction": 736,
 "key-function": 733,
 "keylabel": 737,
 "key-label": 734,
 "keys": 738,
 "KEYWORDS": 739,
 "KEYWORDS-all": 740,
 "label": 741,
 "label-bgcolor": 742,
 "label-dcolor": 743,
 "label-fgcolor": 744,
 "label-font": 745,
 "label-pfcolor": 746,
 "labels": 747,
 "labels-have-colons": 748,
 "landscape": 749,
 "languages": 750,
 "large": 751,
 "large-to-small": 752,
 "last": 753,
 "last-asynch-request": 754,
 "last-batch": 755,
 "last-child": 756,
 "last-event": 757,
 "last-form": 758,
 "lastkey": 765,
 "last-key": 759,
 "last-object": 760,
 "last-of": 761,
 "last-procedure": 762,
 "last-server": 763,
 "last-tab-item": 764,
 "lc": 766,
 "ldbname": 767,
 "le": 18,
 "leave": 768,
 "left-aligned": 769,
 "left-trim": 770,
 "length": 771,
 "library": 772,
 "like": 773,
 "like-sequential": 774,
 "line": 775,
 "line-counter": 776,
 "list-events": 777,
 "listing": 784,
 "list-item-pairs": 778,
 "list-items": 779,
 "list-property-names": 780,
 "list-query-attrs": 781,
 "list-set-attrs": 782,
 "list-widgets": 783,
 "literal-question": 785,
 "little-endian": 786,
 "load": 787,
 "load-domains": 788,
 "load-icon": 789,
 "load-image": 790,
 "load-image-down": 791,
 "load-image-insensitive": 792,
 "load-image-up": 793,
 "load-mouse-pointer": 794,
 "load-picture": 795,
 "load-small-icon": 796,
 "local-name": 797,
 "local-version-info": 798,
 "locator-column-number": 799,
 "locator-line-number": 800,
 "locator-public-id": 801,
 "locator-system-id": 802,
 "locator-type": 803,
 "locked": 805,
 "lock-registration": 804,
 "log": 808,
 "log-audit-event": 806,
 "login-expiration-timestamp": 809,
 "login-host": 810,
 "login-state": 811,
 "log-manager": 807,
 "logout": 812,
 "lookahead": 814,
 "lookup": 815,
 "lt": 17,
 "machine-class": 816,
 "mandatory": 817,
 "manual-highlight": 818,
 "map": 819,
 "margin-extra": 820,
 "margin-height-chars": 821,
 "margin-height-pixels": 822,
 "margin-width-chars": 823,
 "margin-width-pixels": 824,
 "mark-new": 825,
 "mark-row-state": 826,
 "matches": 827,
 "max-button": 828,
 "max-chars": 829,
 "max-data-guess": 830,
 "max-height": 831,
 "max-height-chars": 832,
 "max-height-pixels": 833,
 "maximize": 839,
 "maximum": 840,
 "maximum-level": 841,
 "max-rows": 834,
 "max-size": 835,
 "max-value": 836,
 "max-width-chars": 837,
 "max-width-pixels": 838,
 "md5-digest": 842,
 "member": 843,
 "memptr-to-node-value": 845,
 "menubar": 851,
 "menu-bar": 847,
 "menu-item": 848,
 "menu-key": 849,
 "menu-mouse": 850,
 "merge-by-field": 852,
 "message": 853,
 "message-area": 854,
 "message-area-font": 855,
 "message-lines": 856,
 "method": 857,
 "min-button": 858,
 "min-column-width-chars": 859,
 "min-column-width-pixels": 860,
 "min-height-chars": 861,
 "min-height-pixels": 862,
 "minimum": 867,
 "min-size": 863,
 "min-value": 864,
 "min-width-chars": 865,
 "min-width-pixels": 866,
 "modified": 868,
 "modulo": 869,
 "month": 870,
 "mouse": 871,
 "mouse-pointer": 872,
 "movable": 873,
 "move-after-tab-item": 874,
 "move-before-tab-item": 875,
 "move-column": 876,
 "move-to-bottom": 877,
 "move-to-eof": 878,
 "move-to-top": 879,
 "mpe": 880,
 "mtime": 881,
 "multi-compile": 882,
 "multiple": 883,
 "multiple-key": 884,
 "multitasking-interval": 885,
 "must-exist": 886,
 "name": 887,
 "namespace-prefix": 888,
 "namespace-uri": 889,
 "native": 890,
 "ne": 19,
 "needs-appserver-prompt": 891,
 "needs-prompt": 892,
 "new": 893,
 "new-instance": 894,
 "new-row": 895,
 "next": 896,
 "next-column": 897,
 "next-prompt": 898,
 "next-rowid": 899,
 "next-sibling": 900,
 "next-tab-item": 901,
 "next-value": 902,
 "no": 903,
 "no-apply": 904,
 "no-array-message": 905,
 "no-assign": 906,
 "no-attr-list": 907,
 "no-attr-space": 908,
 "no-auto-validate": 909,
 "no-bind-where": 910,
 "no-box": 911,
 "no-console": 912,
 "no-convert": 913,
 "no-convert-3d-colors": 914,
 "no-current-value": 915,
 "no-debug": 916,
 "node-value-to-memptr": 947,
 "no-drag": 917,
 "no-echo": 918,
 "no-empty-space": 919,
 "no-error": 920,
 "no-fill": 921,
 "no-focus": 922,
 "no-help": 923,
 "no-hide": 924,
 "no-index-hint": 925,
 "no-inherit-bgcolor": 926,
 "no-inherit-fgcolor": 927,
 "no-join-by-sqldb": 928,
 "no-labels": 929,
 "no-lobs": 930,
 "no-lock": 931,
 "no-lookahead": 932,
 "no-map": 933,
 "no-message": 934,
 "nonamespace-schema-location": 948,
 "none": 949,
 "no-pause": 935,
 "no-prefetch": 936,
 "normalize": 950,
 "no-row-markers": 937,
 "no-scrollbar-vertical": 938,
 "no-separate-connection": 939,
 "no-separators": 940,
 "not": 951,
 "no-tab-stop": 941,
 "not-active": 952,
 "no-underline": 942,
 "no-undo": 943,
 "no-validate": 944,
 "now": 953,
 "no-wait": 945,
 "no-word-wrap": 946,
 "null": 954,
 "num-aliases": 955,
 "num-buffers": 956,
 "num-buttons": 957,
 "num-columns": 958,
 "num-copies": 959,
 "num-dbs": 960,
 "num-dropped-files": 961,
 "num-entries": 962,
 "numeric": 979,
 "numeric-format": 980,
 "num-fields": 963,
 "num-formats": 964,
 "num-items": 965,
 "num-iterations": 966,
 "num-lines": 967,
 "num-locked-columns": 968,
 "num-messages": 969,
 "num-parameters": 970,
 "num-references": 971,
 "num-replaced": 972,
 "num-results": 973,
 "num-selected-rows": 974,
 "num-selected-widgets": 975,
 "num-tabs": 976,
 "num-to-retain": 977,
 "num-visible-columns": 978,
 "octet-length": 981,
 "of": 982,
 "off": 983,
 "ok": 984,
 "ok-cancel": 985,
 "old": 986,
 "on": 987,
 "on-frame-border": 988,
 "open": 989,
 "opsys": 990,
 "option": 991,
 "or": 992,
 "ordered-join": 993,
 "ordinal": 994,
 "os-append": 995,
 "os-command": 996,
 "os-copy": 997,
 "os-create-dir": 998,
 "os-delete": 999,
 "os-dir": 1000,
 "os-drives": 1001,
 "os-error": 1002,
 "os-getenv": 1003,
 "os-rename": 1004,
 "otherwise": 1005,
 "output": 1006,
 "overlay": 1007,
 "override": 1008,
 "owner": 1009,
 "page": 1010,
 "page-bottom": 1011,
 "paged": 1016,
 "page-number": 1012,
 "page-size": 1013,
 "page-top": 1014,
 "page-width": 1015,
 "parent": 1018,
 "parse-status": 1019,
 "partial-key": 1020,
 "pascal": 1021,
 "password-field": 1022,
 "pathname": 1023,
 "pause": 1024,
 "pbe-hash-algorithm": 1025,
 "pbe-key-rounds": 1026,
 "pdbname": 1027,
 "persistent": 1028,
 "persistent-cache-disabled": 1029,
 "pfcolor": 1030,
 "pixels": 1031,
 "pixels-per-column": 1032,
 "pixels-per-row": 1033,
 "popup-menu": 1034,
 "popup-only": 1035,
 "portrait": 1036,
 "position": 1037,
 "precision": 1038,
 "prefer-dataset": 1039,
 "prepared": 1041,
 "prepare-string": 1040,
 "preprocess": 1042,
 "preselect": 1043,
 "prev": 1044,
 "prev-column": 1045,
 "prev-sibling": 1046,
 "prev-tab-item": 1047,
 "primary": 1048,
 "printer": 1049,
 "printer-control-handle": 1050,
 "printer-hdc": 1051,
 "printer-name": 1052,
 "printer-port": 1053,
 "printer-setup": 1054,
 "private": 1055,
 "private-data": 1056,
 "privileges": 1057,
 "procedure": 1062,
 "procedure-call-type": 1063,
 "procedure-type": 1064,
 "process": 1065,
 "proc-handle": 1058,
 "proc-status": 1059,
 "proc-text": 1060,
 "proc-text-buffer": 1061,
 "profiler": 1066,
 "program-name": 1067,
 "progress": 1068,
 "progress-source": 1069,
 "prompt": 1070,
 "prompt-for": 1071,
 "promsgs": 1072,
 "propath": 1073,
 "protected": 1075,
 "proversion": 1076,
 "proxy": 1077,
 "proxy-password": 1078,
 "proxy-userid": 1079,
 "public": 1080,
 "public-id": 1081,
 "publish": 1082,
 "published-events": 1083,
 "put": 1084,
 "putbyte": 1094,
 "put-byte": 1085,
 "put-double": 1086,
 "put-float": 1087,
 "put-int64": 1088,
 "put-key-value": 1089,
 "put-long": 1090,
 "put-short": 1091,
 "put-string": 1092,
 "put-unsigned-long": 1093,
 "query-close": 1096,
 "query-off-end": 1097,
 "query-open": 1098,
 "query-prepare": 1099,
 "query-tuning": 1100,
 "question": 1101,
 "quit": 1102,
 "quoter": 1103,
 "radio-buttons": 1105,
 "radio-set": 1106,
 "random": 1107,
 "raw-transfer": 1109,
 "rcode-information": 1110,
 "read-available": 1111,
 "read-exact-num": 1112,
 "read-file": 1113,
 "read-json": 1114,
 "read-only": 1115,
 "read-xml": 1116,
 "readkey": 1117,
 "real": 1118,
 "record-length": 1120,
 "recursive": 1122,
 "reference-only": 1123,
 "refresh": 1124,
 "refreshable": 1126,
 "refresh-audit-policy": 1125,
 "register-domain": 1127,
 "release": 1128,
 "remote": 1129,
 "remove-events-procedure": 1130,
 "remove-super-procedure": 1131,
 "repeat": 1132,
 "replace": 1133,
 "replace-selection-text": 1134,
 "reposition": 1135,
 "reposition-backward": 1136,
 "reposition-forward": 1137,
 "reposition-mode": 1138,
 "reposition-to-row": 1139,
 "reposition-to-rowid": 1140,
 "request": 1141,
 "request-info": 1142,
 "reset": 1143,
 "resizable": 1144,
 "resize": 1145,
 "response-info": 1146,
 "restart-row": 1147,
 "restart-rowid": 1148,
 "retain": 1149,
 "retain-shape": 1150,
 "retry": 1151,
 "retry-cancel": 1152,
 "return": 1153,
 "return-inserted": 1154,
 "returns": 1158,
 "return-to-start-dir": 1155,
 "return-value": 1156,
 "return-value-data-type": 1157,
 "reverse-from": 1159,
 "revert": 1160,
 "revoke": 1161,
 "rgb-value": 1162,
 "right-aligned": 1163,
 "right-trim": 1164,
 "r-index": 1104,
 "roles": 1165,
 "round": 1166,
 "routine-level": 1167,
 "row": 1168,
 "row-height-chars": 1169,
 "row-height-pixels": 1170,
 "row-markers": 1171,
 "row-of": 1172,
 "row-resizable": 1173,
 "rule": 1175,
 "run": 1176,
 "run-procedure": 1177,
 "save": 1178,
 "save cache": 1179,
 "save-as": 1180,
 "save-file": 1181,
 "sax-complete": 1182,
 "sax-parse": 1183,
 "sax-parse-first": 1184,
 "sax-parse-next": 1185,
 "sax-parser-error": 1186,
 "sax-running": 1187,
 "sax-uninitialized": 1188,
 "sax-write-begin": 1189,
 "sax-write-complete": 1190,
 "sax-write-content": 1191,
 "sax-write-element": 1192,
 "sax-write-error": 1193,
 "sax-write-idle": 1194,
 "sax-writer": 1196,
 "sax-write-tag": 1195,
 "schema": 1197,
 "schema-location": 1198,
 "schema-marshal": 1199,
 "schema-path": 1200,
 "screen": 1201,
 "screen-io": 1202,
 "screen-lines": 1203,
 "screen-value": 1204,
 "scroll": 1205,
 "scrollable": 1212,
 "scrollbar-horizontal": 1213,
 "scroll-bars": 1206,
 "scrollbar-vertical": 1214,
 "scroll-delta": 1207,
 "scrolled-row-position": 1215,
 "scrolling": 1216,
 "scroll-offset": 1208,
 "scroll-to-current-row": 1209,
 "scroll-to-item": 1210,
 "scroll-to-selected-row": 1211,
 "sdbname": 1217,
 "seal": 1218,
 "seal-timestamp": 1219,
 "search": 1220,
 "search-self": 1221,
 "search-targer": 1222,
 "section": 1223,
 "security-policy": 1224,
 "seek": 1225,
 "select": 1226,
 "selectable": 1232,
 "select-all": 1227,
 "selected": 1233,
 "select-focused-row": 1228,
 "selection": 1234,
 "selection-end": 1235,
 "selection-list": 1236,
 "selection-start": 1237,
 "selection-text": 1238,
 "select-next-row": 1229,
 "select-prev-row": 1230,
 "select-row": 1231,
 "self": 1239,
 "send": 1240,
 "send-sql-statement": 1241,
 "sensitive": 1242,
 "separate-connection": 1243,
 "separator-fgcolor": 1244,
 "separators": 1245,
 "serializable": 1246,
 "serialize-hidden": 1247,
 "serialize-name": 1248,
 "server": 1249,
 "server-connection-bound": 1250,
 "server-connection-bound-request": 1251,
 "server-connection-context": 1252,
 "server-connection-id": 1253,
 "server-operating-mode": 1254,
 "session": 1255,
 "session-id": 1256,
 "set": 1257,
 "set-appl-context": 1258,
 "set-attr-call-type": 1259,
 "set-attribute-node": 1260,
 "set-blue-value": 1261,
 "set-break": 1262,
 "set-buffers": 1263,
 "set-callback": 1264,
 "set-client": 1265,
 "set-commit": 1266,
 "set-contents": 1267,
 "set-current-value": 1268,
 "set-db-client": 1269,
 "set-dynamic": 1270,
 "set-event-manager-option": 1271,
 "set-green-value": 1272,
 "set-input-source": 1273,
 "set-option": 1274,
 "set-output-destination": 1275,
 "set-parameter": 1276,
 "set-pointer-value": 1277,
 "set-property": 1278,
 "set-red-value": 1279,
 "set-repositioned-row": 1280,
 "set-rgb-value": 1281,
 "set-rollback": 1282,
 "set-selection": 1283,
 "set-size": 1284,
 "set-sort-arrow": 1285,
 "setuserid": 1287,
 "set-wait-state": 1286,
 "sha1-digest": 1288,
 "shared": 1290,
 "share-lock": 1289,
 "show-in-taskbar": 1291,
 "show-stats": 1292,
 "side-label-handle": 1293,
 "side-labels": 1294,
 "signature": 1295,
 "silent": 1296,
 "simple": 1297,
 "single": 1298,
 "single-run": 1299,
 "singleton": 1300,
 "size": 1301,
 "size-chars": 1302,
 "size-pixels": 1303,
 "skip": 1304,
 "skip-deleted-record": 1305,
 "slider": 1306,
 "small-icon": 1307,
 "smallint": 1309,
 "small-title": 1308,
 "some": 1310,
 "sort": 1311,
 "sort-ascending": 1312,
 "sort-number": 1313,
 "source": 1314,
 "source-procedure": 1315,
 "space": 1316,
 "sql": 1317,
 "sqrt": 1318,
 "ssl-server-name": 1319,
 "standalone": 1320,
 "start": 1321,
 "start-document": 1322,
 "start-element": 1323,
 "start-move": 1324,
 "start-resize": 1325,
 "start-row-resize": 1326,
 "state-detail": 1327,
 "static": 1328,
 "status": 1329,
 "status-area": 1330,
 "status-area-font": 1331,
 "stdcall": 1332,
 "stop": 1333,
 "stop-after": 1334,
 "stop-parsing": 1335,
 "stopped": 1336,
 "stored-procedure": 1337,
 "stream-handle": 1339,
 "stream-io": 1340,
 "stretch-to-fit": 1341,
 "strict": 1342,
 "strict-entity-resolution": 1343,
 "string": 1344,
 "string-value": 1345,
 "string-xref": 1346,
 "sub-average": 1347,
 "sub-count": 1348,
 "sub-maximum": 1349,
 "sub-minimum": 1351,
 "subscribe": 1353,
 "substitute": 1354,
 "substring": 1355,
 "sub-total": 1352,
 "subtype": 1356,
 "sum": 1357,
 "super": 1358,
 "super-procedures": 1359,
 "suppress-namespace-processing": 1360,
 "suppress-warnings": 1361,
 "symmetric-encryption-algorithm": 1362,
 "symmetric-encryption-iv": 1363,
 "symmetric-encryption-key": 1364,
 "symmetric-support": 1365,
 "system-alert-boxes": 1366,
 "system-dialog": 1367,
 "system-help": 1368,
 "system-id": 1369,
 "table": 1372,
 "table-handle": 1373,
 "table-number": 1374,
 "table-scan": 1375,
 "tab-position": 1370,
 "tab-stop": 1371,
 "target": 1376,
 "target-procedure": 1377,
 "temp-directory": 1378,
 "temp-table-prepare": 1380,
 "terminal": 1381,
 "terminate": 1382,
 "text": 1383,
 "text-cursor": 1384,
 "text-seg-grow": 1385,
 "text-selected": 1386,
 "then": 1387,
 "this-object": 1388,
 "this-procedure": 1389,
 "thread-safe": 1390,
 "three-d": 1391,
 "throw": 1393,
 "through": 1392,
 "thru": 1394,
 "tic-marks": 1395,
 "time": 1396,
 "time-source": 1397,
 "title": 1398,
 "title-bgcolor": 1399,
 "title-dcolor": 1400,
 "title-fgcolor": 1401,
 "title-font": 1402,
 "to": 1403,
 "today": 1405,
 "toggle-box": 1406,
 "tooltip": 1407,
 "tooltips": 1408,
 "topic": 1411,
 "top-nav-query": 1409,
 "top-only": 1410,
 "to-rowid": 1404,
 "total": 1412,
 "trailing": 1413,
 "trans": 1414,
 "transaction": 1416,
 "transaction-mode": 1417,
 "trans-init-procedure": 1415,
 "transparent": 1418,
 "trigger": 1419,
 "triggers": 1420,
 "trim": 1421,
 "true": 1422,
 "truncate": 1423,
 "type": 1424,
 "type-of": 1425,
 "unbox": 1426,
 "unbuffered": 1427,
 "underline": 1428,
 "undo": 1429,
 "unformatted": 1430,
 "union": 1431,
 "unique": 1432,
 "unique-id": 1433,
 "unique-match": 1434,
 "unix": 1435,
 "unless-hidden": 1436,
 "unload": 1437,
 "unsigned-long": 1438,
 "unsubscribe": 1439,
 "up": 1440,
 "update": 1441,
 "update-attribute": 1442,
 "url": 1443,
 "url-decode": 1444,
 "url-encode": 1445,
 "url-password": 1446,
 "url-userid": 1447,
 "use": 1448,
 "use-dict-exps": 1449,
 "use-filename": 1450,
 "use-index": 1451,
 "user": 1456,
 "use-revvideo": 1452,
 "userid": 1458,
 "user-id": 1457,
 "use-text": 1453,
 "use-underline": 1454,
 "use-widget-pool": 1455,
 "using": 1459,
 "v6display": 1460,
 "v6frame": 1461,
 "validate": 1465,
 "validate-expression": 1466,
 "validate-message": 1467,
 "validate-seal": 1468,
 "validation-enabled": 1469,
 "valid-event": 1462,
 "valid-handle": 1463,
 "valid-object": 1464,
 "value": 1470,
 "value-changed": 1471,
 "values": 1472,
 "verbose": 1474,
 "version": 1475,
 "vertical": 1476,
 "view": 1477,
 "view-as": 1478,
 "view-first-column-on-reopen": 1479,
 "virtual-height-chars": 1480,
 "virtual-height-pixels": 1481,
 "virtual-width-chars": 1482,
 "virtual-width-pixels": 1483,
 "visible": 1484,
 "void": 1485,
 "wait": 1486,
 "wait-for": 1487,
 "warning": 1488,
 "web-context": 1489,
 "weekday": 1490,
 "when": 1491,
 "where": 1492,
 "while": 1493,
 "widget": 1494,
 "widget-enter": 1495,
 "widget-id": 1496,
 "widget-leave": 1497,
 "widget-pool": 1498,
 "width-chars": 1499,
 "width-pixels": 1500,
 "window": 1501,
 "window-maximized": 1502,
 "window-minimized": 1503,
 "window-name": 1504,
 "window-normal": 1505,
 "window-state": 1506,
 "window-system": 1507,
 "with": 1508,
 "word-index": 1509,
 "word-wrap": 1510,
 "work-area-height-pixels": 1511,
 "work-area-width-pixels": 1512,
 "work-area-x": 1513,
 "work-area-y": 1514,
 "write": 1517,
 "write-cdata": 1518,
 "write-characters": 1519,
 "write-comment": 1520,
 "write-data-element": 1521,
 "write-empty-element": 1522,
 "write-entity-ref": 1523,
 "write-external-dtd": 1524,
 "write-fragment": 1525,
 "write-json": 1526,
 "write-message": 1527,
 "write-processing-instruction": 1528,
 "write-status": 1529,
 "write-xml": 1530,
 "write-xmlschema": 1531,
 "x": 1532,
 "xcode": 1534,
 "xml-data-type": 1535,
 "xml-entity-expansion-limit": 1536,
 "xml-node-type": 1537,
 "xml-schema-path": 1538,
 "xml-strict-entity-resolution": 1539,
 "xml-suppress-namespace-processing": 1540,
 "x-of": 1533,
 "xref": 1541,
 "xref-xml": 1542,
 "y": 1543,
 "year": 1545,
 "year-offset": 1546,
 "yes": 1547,
 "yes-no": 1548,
 "yes-no-cancel": 1549,
 "y-of": 1544
}


try:
	from .newcore import JiVi,Thread
except:
	from newcore import JiVi,Thread

J 	 = JiVi
JC   = JiVi.ALL.JiviCore
JAr  = JiVi.ALL.JiviAr
JD 	 = JiVi.ALL.JiviDir
JF   = JiVi.ALL.JiviFile
JIO  = JiVi.ALL.JiviIO
JS   = JiVi.ALL.JiviStr




from jivi.fs import *
from jivi.oe.Helper.Keyword import *
from jivi.oe.Helper.Wrapper import Wrapper
import traceback,sys,re,json
len_KEYWORD_AR = len(KEYWORD_AR)

string_starts = {k : k for k in ['"',"'"]}

CONSTANTS = [
"NONE",
"UNKNOWN",
"KEYWORD",
"STRING",
"STRING_CONCAT",
"NUMBER",
"TABLE",
"FIELD",
"VAR",
"BOOLEAN",
"EXPRESSION",
"PP_UNKNOWN",
"PP_SCOP",
"PP_GLOB",
"PP_UNDEF",
"PP_VAR",
"PP_VAR_CONCAT",
"PP_PAR",
"PP_INCL",
"PP_CONDITION",
"PP_CONDITIONBLOCK",
"COMMAND",
"FILEPATH"]


for i,c in enumerate(CONSTANTS):
	exec(c + " = " + str(i + len_KEYWORD_AR))
	KEYWORD_AR.append(c)
	
	
def Keyword(x):
	s = x.lower().strip()
	return KEYWORD_TAB[s] if s in KEYWORD_TAB else None



DOT = KEYWORD_TAB['.']


def make_readable_ar_helper(ar):
	if isinstance(ar,int): return KEYWORD_AR[ar]
	if not isinstance(ar,list): 
		return ar

	new_ar = []
	for i,xel in enumerate(ar):
		if (not i) and (isinstance(xel,int)):
			new_ar.append(KEYWORD_AR[xel])
		else:
			new_ar.append(make_readable_ar_helper(xel))
			
	
	return new_ar
def make_readable_ar(ar):
	new_ar = []
	for a in ar:
		new_ar.append(make_readable_ar_helper(a))
	return new_ar
	
def print_ar(ar):
	print(make_readable_ar(ar))

def flat_ar(ar):
	new_ar = []
	for a in ar:
	
		if isinstance(a,int):
			new_ar.append(KEYWORD_AR[a])
		else:
			if a[0] == STRING:
				new_ar.append(json.dumps(a[1][0]))
			else:
				new_ar.append(a[1])
	return new_ar

if __name__ == "__main__":
	print("\n".join(["{a} = {i}".format(a=String.minlen(a,50),i=i) for i,a in enumerate(CONSTANTS)]))
	
	


from urllib.request import urlopen,Request

		
def html(url):

	def getBody(txt):
		body = txt
		try:
			tl = txt.lower()
			a = tl.find('<body')
			b = tl.find('>',a+1)
			c = tl.find('</body>',b)
			body = txt[b+1:c]
		except:
			pass
		
		
		return body
	user_agent = 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2272.101 Safari/537.36'
	headers = { 'User-Agent' : user_agent }
	req = Request(url, None, headers)
	resp = urlopen(req)

	html = resp.read()
	try:
		txt = html.decode()
		return getBody(txt)
	except:
		return html



from .menusrc import CoordinateStartStop,Coordinate,LineResponse,InputChar,MenuItemToDisplay
from .menusrc import MenuItem,SearchFilter,Curses,Param,MenuItemCollection,KeyboardEvent,MenuEvent
from .menusrc import MenuStartStringSelect,MouseListener,MenuSearch,Menu


from __future__ import annotations
from typing import List
import inspect
from jivi import jvUtil


def print_shit(*a:List[any]):
	print('-------------')
	for x in a:
		print(x)
	print('-------------\n')




metaDataPropertyName = '_jvMeta'
logPropertyName = 'jvlog'

def do_log(self,level:str,*inp:List[any]):

	to_print = [f'[{self.parent.__class__.__name__} - {level}]'] + [str(self)] +  [str(x) for x in inp]
	print("\n".join(to_print))

class WrapperLogging:
	def __init__(self,jvMeta : WrapperMetaData, parentObject : object):
		self.jvMeta = jvMeta
		self.parent = jvMeta.parent
		self.parent = parentObject
		#print_shit('parentObject',parentObject)
		
		setattr(self.parent,logPropertyName,self)




	def debug(self,*inp:List[any]):
		#return do_log()
		return self.__do_log('debug',*inp)

	def info(self,*inp:List[any]):
		return self.__do_log('info',*inp)

	def warning(self,*inp:List[any]):
		return self.__do_log('warning',*inp)

	def error(self,*inp:List[any]):
		return self.__do_log('error',*inp)

	def critical(self,*inp:List[any]):
		return self.__do_log('critical',*inp)

	def __do_log(self,level:str,*inp:List[any]):

		to_print = [f'[{self.jvMeta.parent.__class__.__name__} - {level}]'] + [str(self.parent)] +  [str(x) for x in inp]

		print_shit('parentObject',"\n".join(to_print))


		#	print("\n".join(to_print))


	def __repr__(self):
		return f'[WrapperLogging]'
	
class WrapperMetaData:
	parent : object
	to_print : List[str]

	def __init__(self,parent):
		self.parent = parent
		self.to_print = []
		self.to_print_ignore_tab = dict()
		self.print_func_tab = dict()
		setattr(parent,metaDataPropertyName,self)

		

	def add_print_method_names(self,*method_names):
		for method_name in method_names:
			self.print_func_tab[method_name] = True	
			self.add_propnames_to_skip(getattr(getattr(self.parent,method_name,{}),'propnames_to_skip',[]))
			
			

	def add_propnames_to_skip(self,propnames_to_skip:List[str]=[]):
		for propname_to_skip in propnames_to_skip:
			self.to_print_ignore_tab[propname_to_skip] = True

	def init_print_funcs(self):
		self.add_print_method_names(*[method_name for method_name, method in self.parent.__dict__.items() if hasattr(method, "print_func")])

		
		
		metaData = self
		def replace_method(method_name):
			old_method = getattr(self.parent,method_name,None)
			if old_method is None:
				print(f'old_method is none {method_name}')
				return None
			
			def replacement(self,*a,**b):
				old_keys = jvUtil.ob.properties_keys_tab(self)
				
				

				if method_name == '__init__':
					WrapperLogging(metaData,self)
					
				retval = old_method(self,*a,**b)
				metaData.add_to_print(
					new_properties := [propname for propname in jvUtil.ob.properties_keys_ar(self) if not propname in old_keys]
				)
				return retval
	
			return replacement

		for method_name in self.print_func_tab.keys():
			setattr(self.parent,method_name,replace_method(method_name))

	def add_to_print(self,ar):
		for x in filter(lambda el : el not in self.to_print_ignore_tab,ar):
			self.to_print_ignore_tab[x] = True
			self.to_print.append(x)

	def class_to_string(self,ob):
		to_print_ar = [[f'[{self.parent.__name__}]','','']]
		for k in self.to_print:
			s_key = str(k)
			s_val = str(getattr(ob,k,None))
			to_print_ar.append(['',s_key,s_val])
		
		max_lens = []
		for i,a in enumerate(to_print_ar):
			if not i:
				max_lens = [len(a_el) for a_el in a]
			else:
				max_lens = [max(max_lens[a_ind],len(a[a_ind])) for a_ind in range(len(a))]

		to_print_ar_next_step = []

		for ar in to_print_ar:
			to_print_ar_next_step.append(' '.join([jvUtil.string.fill_to_len(s,min_len=max_lens[index],left_side=False) for index,s in enumerate(ar)]))
		
		return "\n".join(to_print_ar_next_step)

	@classmethod
	def get_meta_data(cls,ob,create=False) -> WrapperMetaData:
		if (ret_val := getattr(ob,metaDataPropertyName,None)) is not None:
			return ret_val
		
		if create:
			
			return WrapperMetaData(ob)
		return None
	

class jvWrapperSuper:
	_jvMeta:WrapperMetaData
	jvlog : WrapperLogging


	


def Wrapper(propnames_to_skip:List[str]=[]):
	def method_decorator(view):
		view.print_func = True
		view.propnames_to_skip = propnames_to_skip
		return view

	def class_decorator(old_class):
		if WrapperMetaData.get_meta_data(old_class):
			return old_class

		data = WrapperMetaData.get_meta_data(old_class,create=True)
		data.add_print_method_names('__init__')
		data.add_propnames_to_skip(propnames_to_skip)
		data.init_print_funcs()

		class JivfoWrapperClass(old_class):
			_jvMeta:WrapperMetaData
			jvlog : WrapperLogging
			def __repr__(self):
				return self._jvMeta.class_to_string(self)

		return JivfoWrapperClass

	def decorator(old):
		return class_decorator(old) if inspect.isclass(old) else method_decorator(old)
	
	return decorator



from jivi.oe.constants import *

from jivi.fs import *
	
def remove_from_list(ar,to_remove):
	for i in sorted(to_remove,reverse=1):
		ar = ar[:i] + ar[i+1:]
	return ar

INPUT_KEYWORD['view-as'] = 1

class Statement:
	maintype = 'statement'
	type = 'statement'
	def __init__(self,ar,window):
		self.window = window
		self.env = self.window.env
		self.ori = ar[:]
		self.ar = ar
		self.tab = {}
		self.pre_init()
		
		
		self.read_images()

		for a in INPUT_KEYWORD.keys():
			self.read_input(a)
			
		self.read_size()
		
	
	def html(self,textra={},war=[]):
		return ''
	def read_input(self,name,keyword=0):
		if not keyword:
			keyword = name
		
		if not Keyword(keyword):
			print(keyword)
			exit()
		keyword = Keyword(keyword)
		for i,a in enumerate(self.ar[:-1]):
			if Keyword(a) == keyword:
				self.set(name,Alias(self.ar[i+1]))
				self.remove(i,i+1)
				return 1
				
	def pre_init(self):
		pass
			
	def read_size(self):


		for i,x in enumerate(self.ar):
			kw = Keyword(x)
			if kw.find('size') != -1:
				self.set('width',wtp(self.ar[i+1],kw.find('pix') != -1))
				self.set('height',htp(self.ar[i+3],kw.find('pix') != -1))
				return self.remove(i,i+1,i+2,i+3)
	def read_images(self):
		read_images_tab = {'image' : 'image','image-up' : 'image','image-down' : 'image-down','image-insensitive' : 'image-insensitive'}
		
		def read_one():
			lar = len(self.ar)
			for i,x in enumerate(self.ar):
				mainkw = Keyword(x)
				if mainkw in read_images_tab:
					to_remove = [i]
					i += 1
					t = {'file' : 0,'from' : []}
					
				
					def do_next(do_remove=1,forced=0):
						nonlocal i,to_remove
						if forced or i == lar: return (mainkw,t,to_remove)
						i += 1
						if i == lar: return (mainkw,t,to_remove)
						if do_remove: to_remove.append(i)
					
					kw = Keyword(self.ar[i])
					if kw == 'file':
						to_remove.append(i)
						if do_next(): return do_next()
						t['file'] = self.ar[i]
						if do_next(0): return do_next()
						kw = Keyword(self.ar[i])
					
					if kw.find('size') != -1:
						to_remove.append(i)
						if do_next(): return do_next()
						t['w'] = wtp(self.ar[i],kw.find('pix') != -1)
						if do_next(): return do_next()
						t['h'] = htp(self.ar[i],kw.find('pix') != -1)
						if do_next(0): return do_next()
						kw = Keyword(self.ar[i])
						
					if kw == 'from':
						to_remove.append(i)
						if do_next(): return do_next()
						t['from'] = self.ar[i]

					
					return do_next(0,1)
			return (0,0,0)
		mainkw,t,to_remove = read_one()

		while mainkw:
			self.remove(*to_remove)
			if not t['from']:
				del t['from']
			self.set(mainkw,t)
			mainkw,t,to_remove = read_one()
	def set(self,k,v):
		self.tab[k] = v
		setattr(self,k,v)
	def remove(self,*a):
		ar = sorted(a,reverse=1)
		for i in ar:
			self.ar = self.ar[:i] + self.ar[i+1:]
		
		
	def remove_keyword(self,kw):
		for i,a in enumerate(self.ar):
			if Keyword(a) == Keyword(kw):
				self.remove(i)
				break
	def update(self,t):
		for a,b in t.items():
			self.set(a,b)
		
class Define(Statement):
	maintype = 'define'
	
	def __init__(self,*a,**b):
		self.view_as = 0
		
		Statement.__init__(self,*a,**b)
		
		
	def pre_init(self):
		self.name = "unknown"
		self.remove_keyword('define')
		self.remove_keyword('no-undo')
		self.read_input('name',self.type)
		self.set('name',self.name.lower())
class Widget(Define):
	type = 'widget'
	def __init__(self,*a,**b):
		Define.__init__(self,*a,**b)

	def pre_init(self):
		Define.pre_init(self)
		
	
class Frame(Widget):
	type = 'frame'
	def __init__(self,*a,**b):
		Widget.__init__(self,*a,**b)
		
			
	def pre_init(self):
		Widget.pre_init(self)
		i = len(self.ar) - 1
		with_ar = []
		while i != -1:
			if self.ar[i] == 'with':
				with_ar = self.ar[i+1:]
				self.ar = self.ar[:i]
				break
			i -= 1
			
		self.widget_ar = self.ar[:]
		self.ar = with_ar[:]
		

		
	def set_widgets(self,wids):
		widgets = []
		
		war = []
		widget = 0
		
		for i,c in enumerate(self.widget_ar):
			cc = c.lower().strip()
			if cc in wids:
				if war:
					widgets.append((widget,war))
				war = []
				widget = wids[cc]
			elif is_string(cc):
				
				if (not i) or ((((i + 1) < len(self.widget_ar)) and (Keyword(self.widget_ar[i+1]) == 'view-as')) and (not Keyword(self.widget_ar[i-1]) in INPUT_KEYWORD)):
					if war:
						widgets.append((widget,war))
					widget = 0
					war = []
			
			if c: war.append(c)
		
		if war:
			widgets.append((widget,war))
		
		self.read_widgets(widgets)
		
		
	def read_widgets(self,widgets):
		self.widgets = []
		def read_pos(war):
			for i,a in enumerate(war):
				if Keyword(a) == 'at':
					to_remove = [i,i+1,i+2,i+3,i+4]
					top_type = Keyword(war[i+1])
					top_val = war[i+2]
					
					left_type = Keyword(war[i+3])
					left_val = war[i+4]
					
					t = dict(left=ctp(left_val,left_type == 'x'),top=rtp(top_val,top_type == 'y'),aligned='normal')
					if ((i + 5) < len(war)) and Keyword(war[i+5]).find('aligned') != -1:
						t['aligned'] = Keyword(war[i+5])
						to_remove.append(i+5)
			
					
					
					return (t,remove_from_list(war,to_remove))
					
			return ({},war)

		for widget,war in widgets:
			t,war = read_pos(war)
			if not t:
				print(war)
				continue
			self.widgets.append((t,widget,war))
		
	def html(self,textra={},war=[]):
		kak = []
		to_html_frame = """<div class = "frame" style="position: relative; width : 100%; height : 100%">{inner}</div>"""
		
		
		for textra,widget,war in self.widgets:
			if widget:
				kak.append(widget.html(textra,war))
				
		return to_html_frame.format(inner="\n".join(kak))
to_html_button_format = """<button name = "{name}" style="position: absolute; left: {left}px; top: {top}px;height: {height}px; width: {width}px;" title = "{label}"></button>"""
class Button(Widget):
	type = 'button'
	def __init__(self,*a,**b):
		Widget.__init__(self,*a,**b)

				
	def html(self,textra={},war=[]):
		t = {}
		tab = {k : v for k,v in self.tab.items()}
		tab.update(textra)
		if 'label' in tab: tab['label'] = nice_string(tab['label'])
		for a in 'label,left,top,height,width,name'.split(','):
			t[a] = tab[a] if a in tab else ''
		return to_html_button_format.format(**t)
class Browse(Widget):
	type = 'browse'
	def __init__(self,*a,**b):
		Widget.__init__(self,*a,**b)
					
class Buffer(Widget):
	type = 'buffer'
	def __init__(self,*a,**b):
		Widget.__init__(self,*a,**b)
					
class Image(Widget):
	type = 'image'
	def __init__(self,*a,**b):
		Widget.__init__(self,*a,**b)
					
class Rectangle(Widget):
	type = 'rectangle'
	def __init__(self,*a,**b):
		Widget.__init__(self,*a,**b)




class Menu(Widget):
	type = 'menu'
	def __init__(self,*a,**b):
		Widget.__init__(self,*a,**b)


class Submenu(Widget):
	type = 'sub-menu'
	def __init__(self,*a,**b):
		Widget.__init__(self,*a,**b)



to_html_fill_in_format = """<input class = 'inp' type = "text" style="position: absolute; left: {left}px; top: {top}px;height: {height}px; width: {width}px;" value = "{label}">"""

format_label = """<label class = 'hidden lbl' for="{name}">{label}</label>"""

format_withlabel = """
<div style="position: absolute; left: {left}px; top: {top}px">
  {label}
  {input}
</div>
"""


format_input = """<input id = "{name}" type = "text" style="height: {height}px; width: {width}px;" value = "{initial}">"""

def create_tab(ar,t):
	tab = {}
	for a in ar:
		tab[a] = t[a] if a in t else ''
	return tab

class Variable(Widget):
	type = 'variable'
	def __init__(self,*a,**b):
		Widget.__init__(self,*a,**b)

	def pre_init(self):
		Widget.pre_init(self)
		self.read_input('datatype','as')
		self.read_input('name','var')
	def html(self,textra={},war=[]):

		tab = {k : v for k,v in self.tab.items()}
		tab.update(textra)
		if 'label' in tab: tab['label'] = nice_string(tab['label'])


		t = create_tab('left,top'.split(','),tab)
		t['input'] = format_input.format(**create_tab('name,height,width,initial'.split(','),tab))
		t['label'] = format_label.format(**create_tab('name,label'.split(','),tab))
		
		return format_withlabel.format(**t)
class Query(Define):
	type = 'query'
	def __init__(self,*a,**b):
		Define.__init__(self,*a,**b)



class Parameter(Define):
	type = 'parameter'
	def __init__(self,*a,**b):
		Define.__init__(self,*a,**b)


class Temptable(Define):
	type = 'temp-table'
	def __init__(self,*a,**b):
		Define.__init__(self,*a,**b)

class Worktable(Define):
	type = 'work-table'
	def __init__(self,*a,**b):
		Define.__init__(self,*a,**b)

class Workfile(Define):
	type = 'workfile'
	def __init__(self,*a,**b):
		Define.__init__(self,*a,**b)

class Dataset(Define):
	type = 'dataset'
	def __init__(self,*a,**b):
		Define.__init__(self,*a,**b)



class Stream(Define):
	type = 'stream'
	def __init__(self,*a,**b):
		Define.__init__(self,*a,**b)
		

class Field:
	type = 'field'
	def __init__(self,t):
		self.tab = t
		for a,b in t.items():
			setattr(self,a,b)